-- ==============================================================
-- FILE : HE_ADC.VHD  -  DO NOT EDIT THIS FILE
-- ==============================================================
--
-- This file is part of the Hardware Interface Layer (H.I.L) of
-- your project when using your HERON-IO2 module.
--
-- This file must NOT be modified by users of the Module.
--
-- ==============================================================
--
--  Module      : HE_ADC
--  Date        : 07/02/2002
--  Author      : L. Bacquart - ALSE
--  Description : ADC interface, same for the A and B channels
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     L. Bacquart    07-02-02    First Written
--  1.1     R. Williams    26-02-02    Changed names, added known
--                                     I/Os, modified IFD and OFD
--                                     implementation.
--  1.2     R. Williams    08-05-02    Syntax change made to all
--                                     instanciated components,
--                                     (changed '0' to GND)
--                                     for compatibility with
--                                     ModelSim5.5b.
--  2.0     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--  2.3     R. Williams    20-01-03    Changes made to the HSB
--                                     mastering functions, and 
--                                     additional registering added
--                                     to the AE flag to ensure
--                                     correct reading from HEPC8s.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;

-- synopsys translate_off
library UNISIM;
  use UNISIM.vcomponents.all;
-- synopsys translate_on

entity HE_ADC is
  port (
    -- External PADS
    ADC   : in  std_logic_vector(11 downto 0);
    OTR_I : in  std_logic;
    -- Module I/Os
    RST   : in  std_logic;
    SCLK  : in  std_logic;
    DATA  : out std_logic_vector(11 downto 0);
    OTR_O : out std_logic
  );
end HE_ADC;


architecture RTL of HE_ADC is

  component FDC
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic;
      CLR : in  std_logic );
  end component;

  -- Note : attributes on instance are supported by all synthesis tools.

  attribute IOB     : string;
  attribute NODELAY : string;

  attribute IOB of i0  : label is "TRUE";
  attribute IOB of i1  : label is "TRUE";
  attribute IOB of i2  : label is "TRUE";
  attribute IOB of i3  : label is "TRUE";
  attribute IOB of i4  : label is "TRUE";
  attribute IOB of i5  : label is "TRUE";
  attribute IOB of i6  : label is "TRUE";
  attribute IOB of i7  : label is "TRUE";
  attribute IOB of i8  : label is "TRUE";
  attribute IOB of i9  : label is "TRUE";
  attribute IOB of i10 : label is "TRUE";
  attribute IOB of i11 : label is "TRUE";
  attribute IOB of i12 : label is "TRUE";

  attribute NODELAY of i0  : label is "";
  attribute NODELAY of i1  : label is "";
  attribute NODELAY of i2  : label is "";
  attribute NODELAY of i3  : label is "";
  attribute NODELAY of i4  : label is "";
  attribute NODELAY of i5  : label is "";
  attribute NODELAY of i6  : label is "";
  attribute NODELAY of i7  : label is "";
  attribute NODELAY of i8  : label is "";
  attribute NODELAY of i9  : label is "";
  attribute NODELAY of i10 : label is "";
  attribute NODELAY of i11 : label is "";
  attribute NODELAY of i12 : label is "";

begin

  i0  : FDC port map (DATA(0),  D=>ADC(0),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i1  : FDC port map (DATA(1),  D=>ADC(1),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i2  : FDC port map (DATA(2),  D=>ADC(2),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i3  : FDC port map (DATA(3),  D=>ADC(3),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i4  : FDC port map (DATA(4),  D=>ADC(4),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i5  : FDC port map (DATA(5),  D=>ADC(5),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i6  : FDC port map (DATA(6),  D=>ADC(6),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i7  : FDC port map (DATA(7),  D=>ADC(7),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i8  : FDC port map (DATA(8),  D=>ADC(8),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i9  : FDC port map (DATA(9),  D=>ADC(9),  C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i10 : FDC port map (DATA(10), D=>ADC(10), C=>SCLK, CLR=>RST);  -- IOB - NODELAY
  i11 : FDC port map (DATA(11), D=>ADC(11), C=>SCLK, CLR=>RST);  -- IOB - NODELAY

  i12 : FDC port map (Q=>OTR_O, D=>OTR_I,   C=>SCLK, CLR=>RST);  -- IOB - NODELAY

end RTL;

