-- ==============================================================
-- FILE : CTR_DAC.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of Example 3, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : CTR_DAC
--  Date        : 27/02/2002
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : DAC data flow Control
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    27-02-02    First Written
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity CTR_DAC is
  port (
    RST             : in  std_logic;
    FCLK            : in  std_logic;
    DACFIFO_FULL    : in  std_logic;
    DACFIFO_WEN     : out std_logic;
    INFIFO_SEL      : in  std_logic_vector(5 downto 0);
    INFIFO_SINGLE   : in  std_logic_vector(5 downto 0);
    INFIFO_DVALID   : in  std_logic_vector(5 downto 0);
    INFIFO_READ_REQ : out std_logic_vector(5 downto 0)
  );
end CTR_DAC;


architecture RTL of CTR_DAC is

  signal DVALID : std_logic;

begin

  INFIFO_READ_REQ(0) <= INFIFO_SINGLE(0) AND (NOT DACFIFO_FULL) AND INFIFO_SEL(0);
  INFIFO_READ_REQ(1) <= INFIFO_SINGLE(1) AND (NOT DACFIFO_FULL) AND INFIFO_SEL(1);
  INFIFO_READ_REQ(2) <= INFIFO_SINGLE(2) AND (NOT DACFIFO_FULL) AND INFIFO_SEL(2);
  INFIFO_READ_REQ(3) <= INFIFO_SINGLE(3) AND (NOT DACFIFO_FULL) AND INFIFO_SEL(3);
  INFIFO_READ_REQ(4) <= INFIFO_SINGLE(4) AND (NOT DACFIFO_FULL) AND INFIFO_SEL(4);
  INFIFO_READ_REQ(5) <= INFIFO_SINGLE(5) AND (NOT DACFIFO_FULL) AND INFIFO_SEL(5);

  DVALID <= (INFIFO_DVALID(0) AND INFIFO_SEL(0)) OR
            (INFIFO_DVALID(1) AND INFIFO_SEL(1)) OR
            (INFIFO_DVALID(2) AND INFIFO_SEL(2)) OR
            (INFIFO_DVALID(3) AND INFIFO_SEL(3)) OR
            (INFIFO_DVALID(4) AND INFIFO_SEL(4)) OR
            (INFIFO_DVALID(5) AND INFIFO_SEL(5));

  DACFIFO_WEN <= DVALID AND (NOT DACFIFO_FULL);

end RTL;
