-- ==============================================================
-- FILE : DAC_DATA.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of Example 3, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : DAC_DATA
--  Date        : 27/02/2002
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : DAC data for 1 or 2 channels
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    27-02-02    First Written
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity DAC_DATA is
    Port ( RST           : in  std_logic;
           SCLK          : in  std_logic;
           DAC_FIFO      : in  std_logic_vector(31 downto 0);
           DACFIFO_EMPTY : in  std_logic;
           D2A_2CH       : in  std_logic;
           DACFIFO_REN   : out std_logic;
           DAC_A         : out std_logic_vector(13 downto 0);
           DAC_B         : out std_logic_vector(13 downto 0)
           );
end DAC_DATA;

architecture RTL of DAC_DATA is

signal MUX_A    : std_logic_vector(13 downto 0);
signal MUX_B    : std_logic_vector(13 downto 0);
signal READ     : std_logic;
signal EN       : std_logic;
signal SEL_A    : std_logic;

begin

  DACFIFO_REN <= READ AND (NOT DACFIFO_EMPTY);

  process (RST, SCLK)
  begin
    if RST='1' then
      READ <= '1';
    elsif rising_edge(SCLK) then
      READ <= NOT (READ AND (NOT DACFIFO_EMPTY) AND (NOT D2A_2CH));
    end if;
  end process;

  EN <= READ AND (NOT DACFIFO_EMPTY);

  process(SCLK)
  begin
    if rising_edge(SCLK) then
      if EN='1' then
         MUX_A <= DAC_FIFO(13 downto 0);
         MUX_B <= DAC_FIFO(29 downto 16);
      end if;
    end if;
  end process;

  -- DAC_A Mux
  SEL_A <= READ AND (NOT D2A_2CH);
  DAC_A <= MUX_B when SEL_A='1'   else MUX_A;

  -- DAC_B Mux
  DAC_B <= MUX_B when D2A_2CH='1' else (others=>'0');

end RTL;
