/************************************************************/
/*                                                          */
/* THIS IS A DEMO PROGRAM FOR THE HUNT ENGINEERING          */
/* HERON-IO2 board when using the example3 bitstream.       */
/* it is example3b, which takes a large buffer generated in */
/* SDRAM and plays it to the DACs of the HERON-IO2          */
/* THE PROGRAM is for a C6000 module.                       */
/*                                                          */
/*                                                          */
/*                                                          */
/*     PW    ver 1.0    4/10/01   first written             */
/*                                                          */   
/************************************************************/


/* YOU need to set this value to reflect how your HERON-IO2 is connected
to your HERON DSP module. */
#define OUTFIFONO  2       /* FIFO through which IO2 Data is sent */

#include <stdio.h>
#include <stdlib.h>
#include <std.h>
#include <math.h>


/* Use HERON-API for heron1 so you need the next include line*/
#include "heron4.h"

 
/* use unpack library for channel unpacking */
#include "HEL_Unpack.h"                            

#define NO_OP_CHANNELS 2

#define FPGA_TO_DSP 3
#define DSP_TO_FPGA 3
#define FPGA_SLOT	2  
#define USER_WRITE			8
#define USER_READ			9      
#define USER_READ_REPLY		10 
 
/* as it says number of samples per channel*/                            
#define SAMPLES_PER_OP_CHAN 1024 

#define PI 3.1415926535         /* So we can calculate angles in radians */  


#define MALLOC_SIZE 2*1024*1024  /* 8Mbytes*/

short outputs[SAMPLES_PER_OP_CHAN*NO_OP_CHANNELS];                                          
       
#define BUFFER_SIZE_WORDS  SAMPLES_PER_OP_CHAN*NO_OP_CHANNELS/2 


HeronFIFO *wfifo;   	/* Handle for write FIFO */        
HeronHSB *hsb;          /* handle for HSB accesses */
int board,slot;


        
/********************************************************************************/
/*																				*/
/*	readback_user																*/
/*																				*/
/*	This function is used to read back from the FPGA							*/
/*																				*/
/********************************************************************************/
int readback_user(unsigned short targetslot, unsigned char *reply, int buffer_size, int user_addr)
{
	int message_type;
	unsigned char send_data[5];   
	unsigned int done;
	int status;

	send_data[0]=user_addr;
	send_data[1]=1;


   	status = HeronHsbSendMessage(hsb,USER_READ,board,targetslot,send_data,2);  
   	status = HeronHsbReceiveMessage(hsb,&message_type,board,targetslot,reply,buffer_size,&done);
	if (status == HERON_OK) 
	{
		if (message_type != USER_READ_REPLY) 
		{
			printf("SLOT%d:Error: expected user read reply   \n",slot);
			return(-1);
		}
		else 
		{       
			if (done!=1)
			{
				printf("SLOT%d:Error: expected one byte, recieved %d  \n",slot,done);
				return(-1);
			}
			else
			{	
			return(reply[0]);
			}
		}
	}

	return(status);
}    



/****************************************************/
/* makes an HSB message to send to the FPGA module	*/
/* it is a "user" type message to write to the      */
/* application FPGA. In the example1 bitstream address */
/* 0 sets the FIFO to use for input to the FPGA, and */
/* address 1 sets the FIFO to use for writes from the */
/* FPGA												*/
/************************************/


void SendUserData(unsigned short targetslot, int Address, int Data)
{
unsigned char user[2];
unsigned char rec_data[3];
int reply;
int status;

	/* Send user data */
	user[0]=Address;
	user[1]=Data;
        
	status = HeronHsbSendMessage(hsb,USER_WRITE,board,targetslot,user,2);        
           
	if(status != HERON_OK) 
		{
		printf("SLOT%d:HSB Write module failed with %x, exiting\n",targetslot, status);  
		exit(0);
		}
/* read it back to check*/        
    reply =readback_user(targetslot,rec_data,2,Address);
    if(reply!=Data)
    {
    printf("SLOT%d:error hsb readback failed with %x\n",targetslot,reply); 
    exit(0);
    }

}         
      
/****************************************************/
/* makes an HSB message to send to the FPGA module	*/
/* it is a "user" type message to write to the      */
/* application FPGA. In the example1 bitstream address */
/* 0 sets the FIFO to use for input to the FPGA, and */
/* address 1 sets the FIFO to use for writes from the */
/* FPGA												*/
/************************************/


void SendUserDatanr(unsigned short targetslot, int Address, int Data)
{
unsigned char user[2];
int status;

	/* Send user data */
	user[0]=Address;
	user[1]=Data;
        
	status = HeronHsbSendMessage(hsb,USER_WRITE,board,targetslot,user,2);        
         
	if(status != HERON_OK) 
		{
		printf("SLOT%d:HSB Write module failed with %x, exiting\n",targetslot, status);  
		exit(0);
		}

}                  
        
        
main()
{
}        
        


maintask (void)
{
/* WARNING the default stack size for this task ( maintask) defined in    */
/* the cdb file is 8192 bytes                                             */
/* This means that you cannot statically declare more than 2048 integers  */
/* in this task.                                                          */
/* You can increase the stack size allowed by changing the settings for   */
/* the task in the cdb file.                                              */
/* If you need more than 2048 integers, it is better to use Malloc/Free   */
/* to allocate your data buffers 					  */

int status;
int temp;
int i;
int *stored;
short *shortstored;
int *startstored; /* for testing maximum rates */
int done=0;


    printf("Hello from HERON-IO2 Example3b demo.\n");

slot = HeronModId() && 0xf;
board= (HeronModId() && 0xf0)>>4;            
/* open the HSB device to be able to set the FIFO number for the FPGA example1 */
hsb = HeronHsbOpen();
if (hsb == NULL) {
	printf("SLOT%d:Error: Call to HeronHsbOpen failed, error = %d\n",slot,heronerr); 
	exit(0);
}       


{ 
/* open the FIFO for writing*/
    wfifo=HeronOpenFifo(OUTFIFONO,"wd");
    if (wfifo==NULL)
     {
       printf("error opening FIFO %d, error is %x\n",OUTFIFONO,heronerr);
     }
}

stored= malloc(MALLOC_SIZE*sizeof(int)); 
if (stored==NULL)
	{
	printf("malloc failed\n");
	exit(0);
	}
startstored=stored; /* for the record where we started*/
shortstored=(short *)stored;

	temp=1<<DSP_TO_FPGA;  
	SendUserData(FPGA_SLOT, 0, temp); /* set the FIFO the FPGA will use */
#if (NO_OP_CHANNELS==2)	
	SendUserDatanr(FPGA_SLOT, 3, 1); /* set 2 channels operation */
printf("pre-calculating the array for 2 channels\n");	
for(i=0;i<=(MALLOC_SIZE*2)-2;i+=2)
{
	shortstored[i]=(short)(0x1fff+ 0x1fff*( sin(i*(PI/32))));
	shortstored[i+1]=(short)(0x1fff+ 0x1fff*( cos(i*(PI/32)))); 
//	printf("i is %x \n",i);
}		
#else
	SendUserDatanr(FPGA_SLOT, 3, 0); /* set 1 channels operation */
printf("pre-calculating the array for 1 channel\n");	
for(i=0;i<(MALLOC_SIZE*2);i++)
{
	shortstored[i]=(short)(0x1fff+ 0x1fff*( sin(i*(PI/256))));
}
#endif
printf("finished pre-calculating the array\n");	


    /* RELEASE THE CONFIG LINE */
    HeronConfigOff();


while(1)
{
done=0;
stored=startstored;
while(done<=(MALLOC_SIZE-BUFFER_SIZE_WORDS))    
    {   
	/* start to output data*/
	status = HeronWrite(wfifo,stored,(BUFFER_SIZE_WORDS));         
	if (status != HERON_IO_IN_PROGRESS)
		{
		printf("error from write \n");
		exit(0);
		}		

	/* wait for output buffer to be sent*/
	
	status = HeronWaitIo(wfifo);
	if (status != HERON_OK)
		{
		printf("Cannot write fifo %d. Error %d,\n",OUTFIFONO,heronerr);
		exit(0);
		}		
done+=BUFFER_SIZE_WORDS;
stored+=BUFFER_SIZE_WORDS;
	} 
}    
          
/* we cannot reach here but here's how to close the FIFOs if you did */    
	status = HeronClose(wfifo);
	if (status != HERON_OK)
	{
	printf(" Error closing fifo \n");
	exit(-1);
	}    
    
    
    
    

} /* end maintask */
