/********************************************************************************/
/*																				*/
/* example2.c                                	                for heron-io2   */
/*    																			*/
/* Example2 for a HERON-IO2 module used with a DSP module.             	        */
/* It works with the example2 bistream loaded into a HERON-IO2	 				*/
/*																				*/
/*	   2001 Hunt Engineering													*/
/*	    All Rights Reserved														*/
/*																				*/
/* 	Ver	Date		Author	Comments											*/
/*	---	----		------	--------											*/
/*																				*/
/*	1.0	26/6/01	PW		Created.											*/
/********************************************************************************/


#include <stdio.h>
#include <std.h>
#include <swi.h>
#include <tsk.h>  
#include <math.h>
#include "heron4.h"

#pragma FUNC_EXT_CALLED(maintask)


/************************************/
/* FIFO information for Heron_API   */
/************************************/
#define FIFONO	2		/* FIFO through which module communicates with FPGA */
HeronFIFO *rfifo;		/* Handle for read FIFO  */
HeronFIFO *wfifo;   	/* Handle for write FIFO */        
HeronHSB *hsb;          /* handle for HSB accesses */

extern void HEL_UnpackSe16bit2chan (short *input, unsigned short bitmask, short *output, int sam_per_chan, short *errors);

/************************************/
/* Arrays for data in FPGA test     */
/************************************/
#define ARRAY_SIZE 128	/*number of words to write to FPGA*/
#define DATA_MASK 0xfff  
#define OTR_MASK 0x1000
#define UDR_MASK 0x2000
#define SAMPLES_PER_CHAN ARRAY_SIZE
                                 
#define FPGA_TO_DSP 3
#define FPGA_SLOT	2  
#define USER_WRITE			8
#define USER_READ			9      
#define USER_READ_REPLY		10 
  
#define PI 3.1415926535         /* So we can calculate angles in radians */  
  
short *shortdata;		
int *readdata;			/* results read back from FPGA */	
int *chadata;	
int *chbdata;	

int i,slot,board;
  
/* allocate the errors array */
#pragma DATA_ALIGN(errors,4);   
short errors[2];     
  
   
/********************************************************************************/
/*																				*/
/*	readback_user																*/
/*																				*/
/*	This function is used to read back from the FPGA							*/
/*																				*/
/********************************************************************************/
int readback_user(unsigned short targetslot, unsigned char *reply, int buffer_size, int user_addr)
{
	int message_type;
	unsigned char send_data[5];   
	unsigned int done;
	int status;

	send_data[0]=user_addr;
	send_data[1]=1;


   	status = HeronHsbSendMessage(hsb,USER_READ,board,targetslot,send_data,2);  
   	status = HeronHsbReceiveMessage(hsb,&message_type,board,targetslot,reply,buffer_size,&done);
	if (status == HERON_OK) 
	{
		if (message_type != USER_READ_REPLY) 
		{
			printf("SLOT%d:Error: expected user read reply   \n",slot);
			return(-1);
		}
		else 
		{       
			if (done!=1)
			{
				printf("SLOT%d:Error: expected one byte, recieved %d  \n",slot,done);
				return(-1);
			}
			else
			{	
			return(reply[0]);
			}
		}
	}

	return(status);
}    



/****************************************************/
/* makes an HSB message to send to the FPGA module	*/
/* it is a "user" type message to write to the      */
/* application FPGA. In the example1 bitstream address */
/* 0 sets the FIFO to use for input to the FPGA, and */
/* address 1 sets the FIFO to use for writes from the */
/* FPGA												*/
/************************************/


void SendUserData(unsigned short targetslot, int Address, int Data)
{
unsigned char user[2];
unsigned char rec_data[3];
int reply;
int status;

	/* Send user data */
	user[0]=Address;
	user[1]=Data;
        
	status = HeronHsbSendMessage(hsb,USER_WRITE,board,targetslot,user,2);        
        
//    printf("sent\n");    
	if(status != HERON_OK) 
		{
		printf("SLOT%d:HSB Write module failed with %x, exiting\n",targetslot, status);  
		exit(0);
		}
/* read it back to check*/        
    reply =readback_user(targetslot,rec_data,2,Address);
    if(reply!=Data)
    {
    printf("SLOT%d:error hsb readback failed with %x\n",targetslot,reply); 
    exit(0);
    }

}         
      
/****************************************************/
/* makes an HSB message to send to the FPGA module	*/
/* it is a "user" type message to write to the      */
/* application FPGA. In the example1 bitstream address */
/* 0 sets the FIFO to use for input to the FPGA, and */
/* address 1 sets the FIFO to use for writes from the */
/* FPGA												*/
/************************************/


void SendUserDatanr(unsigned short targetslot, int Address, int Data)
{
unsigned char user[2];
int status;

	/* Send user data */
	user[0]=Address;
	user[1]=Data;
        
	status = HeronHsbSendMessage(hsb,USER_WRITE,board,targetslot,user,2);        
        
//    printf("sent\n");    
	if(status != HERON_OK) 
		{
		printf("SLOT%d:HSB Write module failed with %x, exiting\n",targetslot, status);  
		exit(0);
		}

}          
   
int main(void)
{     
}


/************************************/
/* Maintask - main processing task	*/
/* Performs the test by reading the data 	*/
/************************************/

int maintask(void)
{
/* WARNING the default stack size for this task ( maintask) defined in    */
/* the cdb file is 8192 bytes                                             */
/* This means that you cannot statically declare more than 2048 integers  */
/* in this task.                                                          */
/* You can increase the stack size allowed by changing the settings for   */
/* the task in the cdb file.                                              */
/* If you need more than 2048 integers, it is better to use Malloc/Free   */
/* to allocate your data buffers 					  */



int status,fifo;         



                                    
readdata = (int *) malloc(ARRAY_SIZE * sizeof(int));
shortdata = (short *) malloc(ARRAY_SIZE * sizeof(int));
     
chadata = (int *) malloc(512 * sizeof(int));
chbdata = (int *) malloc(512 * sizeof(int));
if ((readdata==0) || (shortdata==0) || (chadata==0) || (chbdata==0))
	{
	printf("Memory allocation error, exiting...\n");
	exit(0);
	}   
	
slot = HeronModId() && 0xf;
board= (HeronModId() && 0xf0)>>4;            
/* open the HSB device to be able to set the FIFO number for the FPGA example1 */
hsb = HeronHsbOpen();
if (hsb == NULL) {
	printf("SLOT%d:Error: Call to HeronHsbOpen failed, error = %d\n",slot,heronerr); 
	exit(0);
}          
          
/* open the FIFO for reading*/
    rfifo=HeronOpenFifo(FIFONO,"r");
    if (rfifo==NULL)
     {
     printf("Cannot open fifo %d for read. Error %d\n",FIFONO,heronerr);
     exit(-1); 	       
     }

/* open the FIFO for writing*/
    wfifo=HeronOpenFifo(FIFONO,"w");
    if (wfifo==NULL)
     {
     printf("Cannot open fifo %d for write. Error %d\n",FIFONO,heronerr);
     exit(-1); 	
     }                                  
     

     
	fifo=1<<FPGA_TO_DSP;  
	SendUserData(FPGA_SLOT, 0, fifo); 
	SendUserData(FPGA_SLOT, 1, fifo);     


/* Start I/O System */
	HeronConfigOff();

/* write a sin and a cos to the DAC rams in the FPGA*/

for (i=0;i<512;i++)
{
chadata[i]=0x10000 |(int)(0x1fff+ 0x1fff*( sin(i*(PI/256)))); 
chbdata[i]=0x40000 |(int)(0x1fff+ 0x1fff*( cos(i*(PI/256))));          
}

status = HeronWrite(wfifo,chadata,512);
	if (status == HERON_IO_IN_PROGRESS)
		{
		status = HeronWaitIo(wfifo);
		}
	if (status != HERON_OK)
		{
        printf("Cannot write fifo %d. Error %d \n",FIFONO,heronerr);
		exit(0);
		}        
status = HeronWrite(wfifo,chbdata,512);
	if (status == HERON_IO_IN_PROGRESS)
		{
		status = HeronWaitIo(wfifo);
		}
	if (status != HERON_OK)
		{
        printf("Cannot write fifo %d. Error %d \n",FIFONO,heronerr);
		exit(0);
		}    
             
while (1)
{
//	printf("sending request\n");
	SendUserDatanr(FPGA_SLOT, 2 ,(ARRAY_SIZE&0xff)); /* bottom part of count */
	SendUserDatanr(FPGA_SLOT, 3 ,((ARRAY_SIZE>>8)&0xff)); /* top part of count and request to start*/	
 
//	printf("request sent -- now startng the read\n");  
  	status = HeronRead(rfifo,readdata,ARRAY_SIZE);
	if (status != HERON_IO_IN_PROGRESS)
	{      
		printf("error cannot start the read\n");
  		exit(-1);
  	}                  

	status = HeronWaitIo(rfifo);
   	if (status != HERON_OK) 
	{
		printf("Cannot read fifo %d. Error %d\n",FIFONO,heronerr);
		exit(-1);
	} 
	printf("Reading Test Pattern complete\n");		
	
	HEL_UnpackSe16bit2chan ((short *)readdata, DATA_MASK, shortdata ,SAMPLES_PER_CHAN, errors);
					
                   
	          

	if((errors[0] & OTR_MASK)!= 0)
	{                 
	printf("Error channel A out of range\n"); 
	}  
	if((errors[1] & OTR_MASK)!= 0)
	{                 
	printf("Error channel B out of range\n"); 
	}               
	if((errors[0] & UDR_MASK)!= 0)
	{                 
	printf("channel A more than half range\n"); 
	}  
	if((errors[1] & UDR_MASK)!= 0)
	{                 
	printf("channel B more than half used\n"); 
	}               
	
	
	
	
	
	
		                        
	HeronDigioOut(0xf);
}
  
  
  
             
             
status = HeronClose(rfifo);
status = HeronClose(wfifo);    
status = HeronHsbClose(hsb);            
             
             
return(0);
}
