-- ==============================================================
-- FILE : DDC.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of DDC Example ,for the HERON-IO2V2 module.
--
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : DDC
--  Date        : 16/01/2003
--  Author      : T. HOLLIS
--  Description : Digital Down Converter
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     T. Hollis     16/01/2003    First Written
--
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity DDC is
  port (
     RST          : in  std_logic;
     SCLK         : in  std_logic;
     ADC_A        : in  std_logic_vector(11 downto 0);
     OP_SEL       : in  std_logic_vector(2 downto 0);
     DDS_FREQ     : in std_logic_vector(25 downto 0);
     DDS_EN       : in std_logic;
     DDC_OP_I     : out std_logic_vector(13 downto 0);
     DDC_OP_RDY_I : out  std_logic;
     DDC_OP_Q     : out std_logic_vector(13 downto 0);
     DDC_OP_RDY_Q : out  std_logic
  );
end DDC;


architecture RTL of DDC is

  component MIXER
      port (
     RST           : in  std_logic;
     SCLK          : in  std_logic;
     ADC_A         : in  std_logic_vector(11 downto 0);
     MULT_OP_I     : out std_logic_vector(13 downto 0);
     MULT_OP_Q     : out std_logic_vector(13 downto 0);
     SIN_OP        : out std_logic_vector(11 downto 0);
     DDS_FREQ      : in  std_logic_vector(25 downto 0);
     DDS_EN        : in std_logic
     );
  end component;

  component CIC
      port (
    RST            : in  std_logic;
    SCLK           : in  std_logic;
    CIC_IP_I      : in  std_logic_vector(13 downto 0);
    CIC_IP_Q      : in  std_logic_vector(13 downto 0);
    CIC_OP_I       : out std_logic_vector(13 downto 0);
    CIC_OP_RDY_I   : out std_logic;
    CIC_OP_Q       : out std_logic_vector(13 downto 0);
    CIC_OP_RDY_Q   : out std_logic
    );
  end component;

  component FILTERS
      port (
      RST           : in  std_logic;
      SCLK          : in  std_logic;
      FILT_IP_I     : in std_logic_vector(13 downto 0);
      FILT_IP_RDY_I : in std_logic;
      FILT_IP_Q     : in std_logic_vector(13 downto 0);
      FILT_IP_RDY_Q : in std_logic;
      FILT1_OP_I     : out std_logic_vector(13 downto 0);
      FILT1_OP_RDY_I : out std_logic;
      FILT1_OP_Q     : out std_logic_vector(13 downto 0);
      FILT1_OP_RDY_Q : out std_logic;
      FILT2_OP_I     : out std_logic_vector(13 downto 0);
      FILT2_OP_RDY_I : out std_logic;
      FILT2_OP_Q     : out std_logic_vector(13 downto 0);
      FILT2_OP_RDY_Q : out std_logic
    );
  end component;


 signal  xSCLK     : std_logic;
 signal  MOP_I     : std_logic_vector(13 downto 0);
 signal  MOP_Q     : std_logic_vector(13 downto 0);

 signal  CIC_OP_I         : std_logic_vector(13 downto 0);
 signal  CIC_OP_RDY_I     : std_logic;
 signal  CIC_OP_Q         : std_logic_vector(13 downto 0);
 signal  CIC_OP_RDY_Q     : std_logic;

 signal  FILT1_OP_I         : std_logic_vector(13 downto 0);
 signal  FILT1_OP_RDY_I     : std_logic;
 signal  FILT1_OP_Q         : std_logic_vector(13 downto 0);
 signal  FILT1_OP_RDY_Q     : std_logic;

 signal  FILT2_OP_I         : std_logic_vector(13 downto 0);
 signal  FILT2_OP_RDY_I     : std_logic;
 signal  FILT2_OP_Q         : std_logic_vector(13 downto 0);
 signal  FILT2_OP_RDY_Q     : std_logic;

 signal  SIN_OP         : std_logic_vector(11 downto 0);

begin


 xSCLK <= SCLK;

 -- MULTIPLEXOR FOR PHASE DATA
         process (OP_SEL,ADC_A,MOP_I,CIC_OP_I,FILT1_OP_I,FILT2_OP_I)
    begin
       case OP_SEL is
         when "000"  =>
           DDC_OP_I <= ADC_A & "00";
         when "001"  =>
           DDC_OP_I <= MOP_I;
         when "010"  =>
           DDC_OP_I <= CIC_OP_I;
         when "011"  =>
           DDC_OP_I <= FILT1_OP_I;
         when "100"  =>
           DDC_OP_I <= FILT2_OP_I;
        when others =>
           DDC_OP_I <= FILT2_OP_I;
       end case;
   end process;

 -- MULTIPLEXOR FOR QUADRATURE DATA
         process (OP_SEL,SIN_OP,MOP_Q,CIC_OP_Q,FILT1_OP_Q,FILT2_OP_Q)
    begin
       case OP_SEL is
         when "000"  =>
           DDC_OP_Q <= SIN_OP & "00";
         when "001"  =>
           DDC_OP_Q <= MOP_Q;
         when "010"  =>
           DDC_OP_Q <= CIC_OP_Q;
         when "011"  =>
           DDC_OP_Q <= FILT1_OP_Q;
         when "100"  =>
           DDC_OP_Q <= FILT2_OP_Q;
        when others =>
           DDC_OP_Q <= FILT2_OP_Q;
       end case;
    end process;

 -- MULTIPLEXOR FOR PHASE DATA READY
         process (OP_SEL,SCLK,CIC_OP_RDY_I,FILT1_OP_RDY_I,FILT2_OP_RDY_I)
    begin
       case OP_SEL is
         when "000"  =>
           DDC_OP_RDY_I <= '1';
         when "001"  =>
           DDC_OP_RDY_I <= '1';
         when "010"  =>
           DDC_OP_RDY_I <= CIC_OP_RDY_I;
         when "011"  =>
           DDC_OP_RDY_I <= FILT1_OP_RDY_I;
         when "100"  =>
           DDC_OP_RDY_I <= FILT2_OP_RDY_I;
        when others =>
           DDC_OP_RDY_I <= FILT2_OP_RDY_I;
       end case;
    end process;

 -- MULTIPLEXOR FOR QUADRATURE DATA READY
       process (OP_SEL,SCLK,CIC_OP_RDY_Q,FILT1_OP_RDY_Q,FILT2_OP_RDY_Q)
    begin
       case OP_SEL is
         when "000"  =>
           DDC_OP_RDY_Q <= '1';
         when "001"  =>
           DDC_OP_RDY_Q <= '1';
         when "010"  =>
           DDC_OP_RDY_Q <= CIC_OP_RDY_Q;
         when "011"  =>
           DDC_OP_RDY_Q <= FILT1_OP_RDY_Q;
         when "100"  =>
           DDC_OP_RDY_Q <= FILT2_OP_RDY_Q;
        when others =>
           DDC_OP_RDY_Q <= FILT2_OP_RDY_Q;
       end case;
    end process;


 iMIXER : MIXER
        port map (
     RST           => RST,
     SCLK          => xSCLK,
     ADC_A         => ADC_A,
     MULT_OP_I     => MOP_I,
     MULT_OP_Q     => MOP_Q,
     SIN_OP        => SIN_OP,
     DDS_FREQ      => DDS_FREQ,
     DDS_EN        => DDS_EN
      );

 iCIC : CIC
        port map (
     RST            => RST,
     SCLK           => xSCLK,
     CIC_IP_I       =>  MOP_I,
     CIC_IP_Q       =>  MOP_Q,
     CIC_OP_I       =>  CIC_OP_I,
     CIC_OP_RDY_I   =>  CIC_OP_RDY_I,
     CIC_OP_Q       =>  CIC_OP_Q,
     CIC_OP_RDY_Q   =>  CIC_OP_RDY_Q
      );

 iFILTERS : FILTERS
        port map (
     RST            => RST,
     SCLK           => xSCLK,
     FILT_IP_I      => CIC_OP_I,
     FILT_IP_RDY_I  => CIC_OP_RDY_I,
     FILT_IP_Q      => CIC_OP_Q,
     FILT_IP_RDY_Q  => CIC_OP_RDY_Q,
     FILT1_OP_I      => FILT1_OP_I,
     FILT1_OP_RDY_I  => FILT1_OP_RDY_I,
     FILT1_OP_Q      => FILT1_OP_Q,
     FILT1_OP_RDY_Q  => FILT1_OP_RDY_Q,
     FILT2_OP_I      => FILT2_OP_I,
     FILT2_OP_RDY_I  => FILT2_OP_RDY_I,
     FILT2_OP_Q      => FILT2_OP_Q,
     FILT2_OP_RDY_Q  => FILT2_OP_RDY_Q
      );


end RTL;
