-- ==============================================================
-- FILE : FILTERS.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of DDC Example ,for the HERON-IO2 module.
-- It contains a pair of FIR filters for both I and Q
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : FILTERS
--  Date        : 21/01/2003
--  Author      : T. HOLLIS
--  Description : Digital Down Converter
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     T. Hollis     16/01/2003    First Written
--
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity FILTERS is
  port (
     RST           : in  std_logic;
     SCLK          : in  std_logic;
     FILT_IP_I     : in std_logic_vector(13 downto 0);
     FILT_IP_RDY_I : in std_logic;
     FILT_IP_Q     : in std_logic_vector(13 downto 0);
     FILT_IP_RDY_Q : in std_logic;
     FILT1_OP_I     : out std_logic_vector(13 downto 0);
     FILT1_OP_RDY_I : out std_logic;
     FILT1_OP_Q     : out std_logic_vector(13 downto 0);
     FILT1_OP_RDY_Q : out std_logic;
     FILT2_OP_I     : out std_logic_vector(13 downto 0);
     FILT2_OP_RDY_I : out std_logic;
     FILT2_OP_Q     : out std_logic_vector(13 downto 0);
     FILT2_OP_RDY_Q : out std_logic
  );
end FILTERS;


architecture RTL of FILTERS is

component filt1
     port (
           ND: IN std_logic;
           RDY: OUT std_logic;
           CLK: IN std_logic;
           RFD: OUT std_logic;
           DIN: IN std_logic_VECTOR(13 downto 0);
           DOUT: OUT std_logic_VECTOR(31 downto 0));
end component;



component filt2
     port (
           ND: IN std_logic;
           RDY: OUT std_logic;
           CLK: IN std_logic;
           RFD: OUT std_logic;
           DIN: IN std_logic_VECTOR(13 downto 0);
           DOUT: OUT std_logic_VECTOR(31 downto 0));
end component;

  signal ND1_I           : std_logic;
  signal OP_RDY_I        : std_logic;
  signal RFD1_I          : std_logic;
  signal DOUT1_I         : std_logic_VECTOR(31 downto 0);

  signal ND1_Q           : std_logic;
  signal OP_RDY_Q        : std_logic;
  signal RFD1_Q          : std_logic;
  signal DOUT1_Q         : std_logic_VECTOR(31 downto 0);

  signal ND2_I           : std_logic;
  signal RFD2_I          : std_logic;
  signal DOUT2_I         : std_logic_VECTOR(31 downto 0);
  signal FILT2_IP_I      : std_logic_VECTOR(13 downto 0);

  signal ND2_Q           : std_logic;
  signal RFD2_Q          : std_logic;
  signal DOUT2_Q         : std_logic_VECTOR(31 downto 0);
  signal FILT2_IP_Q      : std_logic_VECTOR(13 downto 0);

begin


    --PHASE FILTERS
    ---------------

  ND1_I <= FILT_IP_RDY_I AND RFD1_I;

iFILT1_I : filt1
     port map (
               ND => ND1_I,
               RDY => OP_RDY_I,
               CLK => SCLK,
               RFD => RFD1_I,
               DIN => FILT_IP_I,
               DOUT => DOUT1_I);

   -- TRUNCATE TO 14 BITS FOR FILT2
   --and shift a bit
    FILT2_IP_I(13) <= DOUT1_I(31);
    FILT2_IP_I(12 downto 0) <= DOUT1_I(29 downto 17);

    FILT1_OP_I <= FILT2_IP_I;

   ND2_I <= OP_RDY_I AND RFD2_I;

   FILT1_OP_RDY_I <= OP_RDY_I;

iFILT2_I : filt2
     port map (
               ND => ND2_I,
               RDY => FILT2_OP_RDY_I,
               CLK => SCLK,
               RFD => RFD2_I,
               DIN => FILT2_IP_I,
               DOUT => DOUT2_I);

    -- TRUNCATE TO 14 BITS FOR DAC'S
    --and shift 3 bits
    FILT2_OP_I(13) <= DOUT2_I(31);
    FILT2_OP_I(12 downto 0) <= DOUT2_I(27 downto 15);


    --QUADRATURE FILTERS
    --------------------

    ND1_Q <= FILT_IP_RDY_Q AND RFD1_Q;

iFILT1_Q : filt1
     port map (
               ND => ND1_Q,
               RDY => OP_RDY_Q,
               CLK => SCLK,
               RFD => RFD1_Q,
               DIN => FILT_IP_Q,
               DOUT => DOUT1_Q);

   -- TRUNCATE TO 16 BITS FOR FILT2
   --and shift a bit
   FILT2_IP_Q(13) <= DOUT1_Q(31);
   FILT2_IP_Q(12 downto 0) <= DOUT1_Q(29 downto 17);

   FILT1_OP_Q <= FILT2_IP_Q;
 
  ND2_Q <=OP_RDY_Q AND RFD2_Q;

  FILT1_OP_RDY_Q <= OP_RDY_Q;

iFILT2_Q : filt2
     port map (
               ND => ND2_Q,
               RDY => FILT2_OP_RDY_Q,
               CLK => SCLK,
               RFD => RFD2_Q,
               DIN => FILT2_IP_Q,
               DOUT => DOUT2_Q);

    -- TRUNCATE TO 14 BITS FOR DAC'S
    --and shift 3 bits
    FILT2_OP_Q(13) <= DOUT2_Q(31);
    FILT2_OP_Q(12 downto 0) <= DOUT2_Q(27 downto 15);


end RTL;
