
#if _LINUX
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <semaphore.h>
#include <errno.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#else
#include "stdafx.h"
#include "resource.h"
#include "commdlg.h"
#include "stdio.h"
#include "math.h"
#endif

#include "settings.h"

extern int graphscreen;
extern int ymax, ymin;
extern int dofft1, dofft2;

int xmargin=10;
int ymargin=10;

struct graphdata
{
	int autoscale;
	int data_type;
	int min_y_value;
	int max_y_value;
	int hex;
};

extern struct graphdata ChGraphData[2];

typedef struct Complex
{
	float imag;
	float real;
} COMPLEX;

#if _LINUX
void dft(int N, COMPLEX *X);
void fft(int N, COMPLEX *X);
void spdftr(float *x, COMPLEX *y, int N);
#else
extern "C" {
void dft(int N, COMPLEX *X);
void fft(int N, COMPLEX *X);
void spdftr(float *x, COMPLEX *y, int N);
}
#endif

int olddata[4][1024];
int oldsize[4];
static int oldv[4];
static int oldy[4] = { 0, 0, 0, 0 };

#if _LINUX

#if DDC
void DrawAxes(GtkWidget *gDrawArea, GdkPixmap *gPixmap, int x1, int x2, int y, int y1, int y2, int x, int size, int chan)

#else
void DrawAxes(GtkWidget *gDrawArea, GdkPixmap *gPixmap, int x1, int x2, int y, int y1, int y2, int x)
#endif
{
        gdk_draw_line(gPixmap, gDrawArea->style->text_gc[GTK_STATE_SELECTED], x1, y , x2, y );
	gdk_draw_line(gPixmap, gDrawArea->style->text_gc[GTK_STATE_SELECTED], x1, y1, x1, y2);

#if DDC
//	{
//		char msg[256];
//		sprintf(msg, "x2=%d,x1=%d,x2-x1=%d,size=%d", x2, x1, x2-x1, size);
//		MessageBox(NULL, msg, "", MB_OK);
//	}

	if ((chan==2)||(chan==3))
	{
		if (size > (2*(x2-x1)))
		{
		}
		else
		if (size > (x2-x1))
		{
                    gdk_draw_line(gPixmap, gDrawArea->style->text_gc[GTK_STATE_SELECTED], x1+(size/2), y-4, x1+(size/2), y+4);
		}
		else
		{
                    gdk_draw_line(gPixmap, gDrawArea->style->text_gc[GTK_STATE_SELECTED], x1+(size/2), y-4, x1+(size/2), y+4);
                    gdk_draw_line(gPixmap, gDrawArea->style->text_gc[GTK_STATE_SELECTED], x1+ size   , y-4, x1+ size   , y+4);
		}
	}
#endif
}

void DrawOneGraph_si(GtkWidget *gDrawArea, GdkPixmap *gPixmap,
                     int x1, int y1, int x2, int y2, int *data, int size, int chan, int autoscale)
{
	int px, py;

	if (autoscale)
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=data[0];
		min=data[0];
		for (i=1; i<size; i++)
		{
			if (data[i]>max) max=data[i];
			if (data[i]<min) min=data[i];
		}

		// Change all to a relative position ////////////////////////////////////////
		if (min>0) min = 0;
		if (max<0) max = 0;
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

		// Draw the two axes ////////////////////////////////////////////////////////
		if (min<0)
		{
			y = ((float)max/((float)(max-min)))*s+y1;
		}
		else
		{
			y = y2;
		}
#if DDC
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = y;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		if (min<0)
		{
			y   = -(int)(f*(float)(data[0]-min)) + y2;
		}
		else
		{
			y   = -(int)(f*(float)data[0]) + y2;
		}
		//MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]    = size;
		olddata[chan][0] = y;
		oldv   [chan]    = v; 
		px               = x1;
                py               = y;
		if (min<0)
		{
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)(data[i] - min)) + y2;
				//LineTo  (hdc, v*i+x1, y);
				//MoveToEx(hdc, v*i+x1, y, NULL);
				gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
				px = v*i+x1;
				py = y;
				olddata[chan][i] = y;
			}
		}
		else
		{
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)data[i]) + y2;
				//LineTo  (hdc, v*i+x1, y);
				//MoveToEx(hdc, v*i+x1, y, NULL);
				gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
				px = v*i+x1;
				py = y;
				olddata[chan][i] = y;
			}
		}
//		ReleaseDC(hWnd, hdc);
	}
	else
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=ChGraphData[chan].max_y_value;		// use user specified maximum
		min=ChGraphData[chan].min_y_value;		// use user specified minimum
		if (max<min)
		{
			int tmp;
			tmp = max;
			max = min;
			min = tmp;
		}

		// Change all to a relative position ////////////////////////////////////////
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

//		// Draw the graph ///////////////////////////////////////////////////////////
//		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
		if (min<0)
		{
			y = -(int)(f*(float)(ChGraphData[chan].min_y_value-min)) + y2;
		}
		else
		{
			y = -(int)(f*(float)ChGraphData[chan].min_y_value) + y2;
		}
#if DDC
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		if (min<0)
		{
			y = -(int)(f*(float)(data[0]-min)) + y2;
		}
		else
		{
			y = -(int)(f*(float)data[0]) + y2;
		}
		if (y<y1) y=y1;
		else
		if (y>y2) y=y2;
		//MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]   =size;
		oldv   [chan]   =v;
		olddata[chan][0]=y;
		px              =x1;
		py		=y;
		if (min<0)
		{
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)(data[i]-min)) + y2;
				if (y<y1) y=y1;
				else
				if (y>y2) y=y2;
				//LineTo  (hdc, v*i+x1, y);
				//MoveToEx(hdc, v*i+x1, y, NULL);
				gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
				px = v*i+x1;
				py = y;
				olddata[chan][i]=y;
			}
		}
		else
		{
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)data[i]) + y2;
				if (y<y1) y=y1;
				else
				if (y>y2) y=y2;
				//LineTo  (hdc, v*i+x1, y);
				//MoveToEx(hdc, v*i+x1, y, NULL);
				gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
				px = v*i+x1;
				py = y;
				olddata[chan][i]=y;
			}
		}
//		ReleaseDC(hWnd, hdc);
	}
}

void DrawOneGraph_ui(GtkWidget *gDrawArea, GdkPixmap *gPixmap,
                     int x1, int y1, int x2, int y2, unsigned int *data, int size, int chan, int autoscale)
{
	int px, py;

	if (autoscale)
	{
		unsigned int	max, min;
		int				i, y, d, s, v;
		float			f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=data[0];
		min=data[0];
		for (i=1; i<size; i++)
		{
			if (data[i]>max) max=data[i];
			if (data[i]<min) min=data[i];
		}

		// Change all to a relative position ////////////////////////////////////////
		if (min>0) min = 0;
		if (max<0) max = 0;
		d =  max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

//		// Draw the graph ///////////////////////////////////////////////////////////
//		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
#if DDC
		DrawAxes(gDrawArea, gPixmap, x1, x2, y2, y1, y2, x1, size, chan);
#else
		DrawAxes(gDrawArea, gPixmap, x1, x2, y2, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		y   = -(int)(f*(float)data[0]) + y2;
//		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]    = size;
		oldv   [chan]    = v;
		olddata[chan][0] = y;
		px               = x1;
		py               = y;
		for (i=1; (i<size)&&(i<(x2-x1)); i++)
		{
			y = -(int)(f*(float)data[i]) + y2;
//			LineTo  (hdc, v*i+x1, y);
//			MoveToEx(hdc, v*i+x1, y, NULL);
			gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
			px = v*i+x1;
			py = y;
			olddata[chan][i] = y;
		}
//		ReleaseDC(hWnd, hdc);
	}
	else
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=ChGraphData[chan].max_y_value;		// use user specified maximum
		min=ChGraphData[chan].min_y_value;		// use user specified minimum
		if (max<min)
		{
			int tmp;
			tmp = max;
			max = min;
			min = tmp;
		}

		// Change all to a relative position ////////////////////////////////////////
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

//		// Draw the graph ///////////////////////////////////////////////////////////
//		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
		if (max<0) y = y1;
		else
		if (min>0) y = y2;
		else       y = (int)(f*(float)ChGraphData[chan].min_y_value) + y2;
#if DDC
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		y   = -(int)(f*(float)data[0]-f*(float)ChGraphData[chan].min_y_value) + y2;
		if (y<y1) y=y1;
		else
		if (y>y2) y=y2;
//		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]   =size;
		oldv   [chan]   =v;
		olddata[chan][0]=y;
		px              =x1;
		py              =y;
		for (i=1; (i<size)&&(i<(x2-x1)); i++)
		{
			y = -(int)(f*(float)data[i]-f*(float)ChGraphData[chan].min_y_value) + y2;
			if (y<y1) y=y1;
			else
			if (y>y2) y=y2;
//			LineTo  (hdc, v*i+x1, y);
//			MoveToEx(hdc, v*i+x1, y, NULL);
			gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
			px = v*i+x1;
			py = y;
			olddata[chan][i]=y;
		}
//		ReleaseDC(hWnd, hdc);
	}
}

void DrawOneGraph_ss(GtkWidget *gDrawArea, GdkPixmap *gPixmap,
                     int x1, int y1, int x2, int y2, short int *data, int size, int chan, int autoscale)
{
	int px, py;

	if (autoscale)
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=data[0];
		min=data[0];
		for (i=1; i<size; i++)
		{
			if (data[i]>max) max=data[i];
			if (data[i]<min) min=data[i];
		}

		// Change all to a relative position ////////////////////////////////////////
		if (min>0) min = 0;
		if (max<0) max = 0;
		d =  max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

//		// Draw the graph ///////////////////////////////////////////////////////////
//		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
		if (min<0)
		{
			y = ((float)max/((float)(max-min)))*s+y1;
		}
		else
		{
			y = y2;
		}
#if DDC
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = y;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		if (min<0)
		{
			y   = -(int)(f*(float)(data[0]-min)) + y2;
			//MoveToEx(hdc, x1, y, NULL);
			oldsize[chan]    = size;
			oldv   [chan]    = v;
			olddata[chan][0] = y;
			px               = x1;
			py		 = y;
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)(data[i]-min)) + y2;
				//LineTo  (hdc, v*i+x1, y);
				//MoveToEx(hdc, v*i+x1, y, NULL);
				gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
				px = v*i+x1;
				py = y;
				olddata[chan][i] = y;
			}
		}
		else
		{
			y   = -(int)(f*(float)data[0]) + y2;
			//MoveToEx(hdc, x1, y, NULL);
			oldsize[chan]    = size;
			oldv   [chan]    = v;
			olddata[chan][0] = y;
			px               = x1;
                        py               = y;
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)data[i]) + y2;
				//LineTo  (hdc, v*i+x1, y);
				//MoveToEx(hdc, v*i+x1, y, NULL);
				gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
				px = v*i+x1;
				py = y;
				olddata[chan][i] = y;
			}
		}
//		ReleaseDC(hWnd, hdc);
	}
	else
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=ChGraphData[chan].max_y_value;		// use user specified maximum
		min=ChGraphData[chan].min_y_value;		// use user specified minimum
		if (max<min)
		{
			int tmp;
			tmp = max;
			max = min;
			min = tmp;
		}

		// Change all to a relative position ////////////////////////////////////////
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

//		// Draw the graph ///////////////////////////////////////////////////////////
//		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
#if DDC
		DrawAxes(gDrawArea, gPixmap, x1, x2, y2, y1, y2, x1, size, chan);
#else
		DrawAxes(gDrawArea, gPixmap, x1, x2, y2, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		if (min<0)
		{
			y   = -(int)(f*(float)(data[0]-min)) + y2;
			if (y<y1) y=y1;
			else
			if (y>y2) y=y2;
			//MoveToEx(hdc, x1, y, NULL);
			oldsize[chan]    = size;
			oldv   [chan]    = v;
			olddata[chan][0] = y;
			px               = x1;
			py               = y;
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)(data[i]-min)) + y2;
				if (y<y1) y=y1;
				else
				if (y>y2) y=y2;
				//LineTo  (hdc, v*i+x1, y);
				//MoveToEx(hdc, v*i+x1, y, NULL);
				gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
				px = v*i+x1;
				py = y;
				olddata[chan][i]=y;
			}
		}
		else
		{
			y   = -(int)(f*(float)data[0]) + y2;
			if (y<y1) y=y1;
			else
			if (y>y2) y=y2;
			//MoveToEx(hdc, x1, y, NULL);
			oldsize[chan]    = size;
			oldv   [chan]    = v;
			olddata[chan][0] = y;
			px               = x1;
			py               = y;
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)data[i]) + y2;
				if (y<y1) y=y1;
				else
				if (y>y2) y=y2;
				//LineTo  (hdc, v*i+x1, y);
				//MoveToEx(hdc, v*i+x1, y, NULL);
				gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
				px = v*i+x1;
				py = y;
				olddata[chan][i]=y;
			}
		}
//		ReleaseDC(hWnd, hdc);
	}
}

void DrawOneGraph_us(GtkWidget *gDrawArea, GdkPixmap *gPixmap,
                     int x1, int y1, int x2, int y2, unsigned short int *data, int size, int chan, int autoscale)
{
	int px, py;

	if (autoscale)
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=data[0];
		min=data[0];
		for (i=1; i<size; i++)
		{
			if (data[i]>max) max=data[i];
			if (data[i]<min) min=data[i];
		}

		// Change all to a relative position ////////////////////////////////////////
		if (min>0) min = 0;
		if (max<0) max = 0;
		d =  max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

//		// Draw the graph ///////////////////////////////////////////////////////////
//		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
#if DDC
		DrawAxes(gDrawArea, gPixmap, x1, x2, y2, y1, y2, x1, size, chan);
#else
		DrawAxes(gDrawArea, gPixmap, x1, x2, y2, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		y   = -(int)(f*(float)data[0]) + y2;
//		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]    = size;
		oldv   [chan]    = v;
		olddata[chan][0] = y;
		px               = x1;
		py               = y;
		for (i=1; (i<size)&&(i<(x2-x1)); i++)
		{
			y = -(int)(f*(float)data[i]) + y2;
//			LineTo  (hdc, v*i+x1, y);
//			MoveToEx(hdc, v*i+x1, y, NULL);
			gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
			px = v*i+x1;
			py = y;
			olddata[chan][i] = y;
		}
//		ReleaseDC(hWnd, hdc);
	}
	else
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=ChGraphData[chan].max_y_value;		// use user specified maximum
		min=ChGraphData[chan].min_y_value;		// use user specified minimum
		if (max<min)
		{
			int tmp;
			tmp = max;
			max = min;
			min = tmp;
		}

		// Change all to a relative position ////////////////////////////////////////
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

//		// Draw the graph ///////////////////////////////////////////////////////////
//		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
		if (max<0) y = y1;
		else
		if (min>0) y = y2;
		else       y = (int)(f*(float)ChGraphData[chan].min_y_value) + y2;
#if DDC
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(gDrawArea, gPixmap, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		y   = -(int)(f*(float)data[0]-f*(float)ChGraphData[chan].min_y_value) + y2;
		if (y<y1) y=y1;
		else
		if (y>y2) y=y2;
//		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]   =size;
		oldv   [chan]   =v;
		olddata[chan][0]=y;
		for (i=1; (i<size)&&(i<(x2-x1)); i++)
		{
			y = -(int)(f*(float)data[i]-f*(float)ChGraphData[chan].min_y_value) + y2;
			if (y<y1) y=y1;
			else
			if (y>y2) y=y2;
//			LineTo  (hdc, v*i+x1, y);
//			MoveToEx(hdc, v*i+x1, y, NULL);
			gdk_draw_line(gPixmap, gDrawArea->style->black_gc, px, py, v*i+x1, y);
			px = v*i+x1;
			py = y;
			olddata[chan][i]=y;
		}
//		ReleaseDC(hWnd, hdc);
	}
}

void DoDFT_si(int *chdata, int *fftdata, int size)
{
	int		i, max, min;
	float   f, data[1024];
	COMPLEX mydata[1024];

	if (size>1024)
	{
		//MessageBox(NULL, "FFT size must be 1024 or lower.", "FFT", MB_OK);
		printf("FFT size must be 1024 or lower.\n");
		return;
	}

	// Normalise input data: to between -100 and 100
	max = chdata[0];
	min = chdata[0];
	for (i=1; i<size; i++)
	{
		if (chdata[i]>max) max=chdata[i];
		if (chdata[i]<min) min=chdata[i];
	}

	if (min<0)
	{
		if (max==min)
			f = 1.0;
		else
			f = (float) 100.0 / (float) (max-min);
	}
	else
	{
		if (max==0) 
			f = 1.0;
		else
			f = (float) 100.0 / (float) max;
	}

	for (i=0; i<size; i++)
	{
		data[i] = f * (float) (chdata[i]);
	}

	spdftr(data, mydata, size);
	for (i=0; i<size/2; i++)
	{
		fftdata[i] = (int) sqrt(mydata[i].real*mydata[i].real+mydata[i].imag*mydata[i].imag);
	}
}

void DoDFT_ui(unsigned int *chdata, unsigned int *fftdata, int size)
{
	int				i;
	unsigned int	max, min;
	float			f, data[1024];
	COMPLEX			mydata[1024];

	if (size>1024)
	{
		//MessageBox(NULL, "FFT size must be 1024 or lower.", "FFT", MB_OK);
		printf("FFT size must be 1024 or lower.\n");
		return;
	}

	// Normalise input data: to between -100 and 100
	max = chdata[0];
	min = chdata[0];
	for (i=1; i<size; i++)
	{
		if (chdata[i]>max) max=chdata[i];
		if (chdata[i]<min) min=chdata[i];
	}

	if (min<0)
	{
		if (max==min)
			f = 1.0;
		else
			f = (float) 100.0 / (float) (max-min);
	}
	else
	{
		if (max==0) 
			f = 1.0;
		else
			f = (float) 100.0 / (float) max;
	}

	for (i=0; i<size; i++)
	{
		data[i] = f * (float) (chdata[i]);
	}

	spdftr(data, mydata, size);
	for (i=0; i<size/2; i++)
	{
		fftdata[i] = (int) sqrt(mydata[i].real*mydata[i].real+mydata[i].imag*mydata[i].imag);
	}
}

void DoDFT_ss(short *chdata, short *fftdata, int size)
{
	int		i, max, min;
	float   f, data[1024];
	COMPLEX mydata[1024];

	if (size>1024)
	{
		//MessageBox(NULL, "FFT size must be 1024 or lower.", "FFT", MB_OK);
		printf("FFT size must be 1024 or lower.\n");
		return;
	}

	max = chdata[0];
	min = chdata[0];
	for (i=1; i<size; i++)
	{
		if (chdata[i]>max) max=chdata[i];
		if (chdata[i]<min) min=chdata[i];
	}

	if (min<0)
	{
		if (max==min)
			f = 1.0;
		else
			f = (float) 100.0 / (float) (max-min);
	}
	else
	{
		if (max==0) 
			f = 1.0;
		else
			f = (float) 100.0 / (float) max;
	}

	for (i=0; i<size; i++)
	{
		data[i] = f * (float) (chdata[i]);
	}

	spdftr(data, mydata, size);
	for (i=0; i<size/2; i++)
	{
		fftdata[i] = (int) sqrt(mydata[i].real*mydata[i].real+mydata[i].imag*mydata[i].imag);
	}
}

void DoDFT_us(unsigned short *chdata, unsigned short *fftdata, int size)
{
	int		i, max, min;
	float   f, data[1024];
	COMPLEX mydata[1024];

	if (size>1024)
	{
		//MessageBox(NULL, "FFT size must be 1024 or lower.", "FFT", MB_OK);
		printf("FFT size must be 1024 or lower.\n");
		return;
	}

	// Normalise input data: to between -100 and 100
	max = chdata[0];
	min = chdata[0];
	for (i=1; i<size; i++)
	{
		if (chdata[i]>max) max=chdata[i];
		if (chdata[i]<min) min=chdata[i];
	}

	if (min<0)
	{
		if (max==min)
			f = 1.0;
		else
			f = (float) 100.0 / (float) (max-min);
	}
	else
	{
		if (max==0) 
			f = 1.0;
		else
			f = (float) 100.0 / (float) max;
	}

	for (i=0; i<size; i++)
	{
		data[i] = f * (float) (chdata[i]);
	}

	spdftr(data, mydata, size);
	for (i=0; i<size/2; i++)
	{
		fftdata[i] = (int) sqrt(mydata[i].real*mydata[i].real+mydata[i].imag*mydata[i].imag);
	}
}

void DoDFT(int datatype, void *chdata, void *fftdata, int size)
{
	switch (datatype)
	{
	case 0: DoDFT_si((           int *)chdata, (           int *)fftdata, size); break;
	case 1: DoDFT_ss((         short *)chdata, (         short *)fftdata, size); break;
	case 2: DoDFT_ui((unsigned   int *)chdata, (unsigned   int *)fftdata, size); break;
	case 3: DoDFT_us((unsigned short *)chdata, (unsigned short *)fftdata, size); break;
	}
}

void DrawOneGraph(GtkWidget *gDrawArea, GdkPixmap *gPixmap, int x1, int y1, int x2, int y2, void *data, int size, int chan)
{
	static int autoscale, datatype, first = 1;
	int fftdata[1024];

//	// Erase previous graph
//	if (!first)
//	{
//		HPEN pen, old;
//		int  i,v;
//
//		pen = CreatePen(PS_SOLID, 1, GetBkColor(hdc));
//		old = (HPEN)SelectObject(hdc, pen);
//
//		// delete old x-axis - if necessary
//		if (oldy[chan])
//		{
//			MoveToEx(hdc, x1, oldy[chan], NULL);
//			LineTo  (hdc, x2, oldy[chan]);
//		}
//
//		// get old index mutiplier
//		v   = oldv[chan];
//		// move to first data point
//		MoveToEx(hdc, x1, olddata[chan][0], NULL);
//		for (i=1; i<oldsize[chan]; i++)
//		{
//			LineTo  (hdc, v*i+x1, olddata[chan][i]);
//			MoveToEx(hdc, v*i+x1, olddata[chan][i], NULL);
//		}
//		SelectObject(hdc, old);
//		DeleteObject(pen);
//	}
//	first=0;

	switch (chan)
	{
	case 0  : autoscale = ChGraphData[0].autoscale; datatype = ChGraphData[0].data_type; break;
	case 1  : autoscale = ChGraphData[1].autoscale; datatype = ChGraphData[1].data_type; break;
	case 2  : if (!dofft1) return;
                  autoscale = 1;                        datatype = ChGraphData[0].data_type;
                  DoDFT(datatype, data, fftdata, size); data = fftdata; size=size/2;
                  break;
	case 3  : if (!dofft2) return;
                  autoscale = 1;                        datatype = ChGraphData[1].data_type;
                  DoDFT(datatype, data, fftdata, size); data = fftdata; size=size/2;
                  break;
	default : return;
	}

	switch (datatype)
	{
	case 0: DrawOneGraph_si(gDrawArea, gPixmap, x1, y1, x2, y2, (           int *)data, size, chan, autoscale); break;
	case 1: DrawOneGraph_ss(gDrawArea, gPixmap, x1, y1, x2, y2, (         short *)data, size, chan, autoscale); break;
	case 2: DrawOneGraph_ui(gDrawArea, gPixmap, x1, y1, x2, y2, (unsigned   int *)data, size, chan, autoscale); break;
	case 3: DrawOneGraph_us(gDrawArea, gPixmap, x1, y1, x2, y2, (unsigned short *)data, size, chan, autoscale); break;
	}
}

struct rect
{
	int left;
	int top;
	int right;
	int bottom;
};

typedef struct rect RECT;

void DrawGraph(GtkWidget *gDrawArea[4], GdkPixmap *gPixmap[4], void *ch1data, void *ch2data, int size1, int size2)
{
	int         i=0, x1, y1, x2, y2;
	RECT        rt, g1, g2, f1, f2;
	GdkFont    *font;
	GdkColor    blue, red, purple, green;
	GtkRcStyle *rc_style;
	static int  first=1;
//	SIZE Sz, *sz;
//	COLORREF mycolor = RGB(192, 0, 192);

//	sz = &Sz;
//
//	hdc = GetDC(hWnd);
//	if (GetTextExtentPoint32(hdc, "A", 1, sz)==0)
	{
		xmargin =  8;
		ymargin = 16;
	}
//	else
//	{
//		char msg[256];
//		sprintf(msg, "cx=%d, cy=%d", sz->cx, sz->cy);
////		MessageBox(NULL, msg, "sz", MB_OK);
//		xmargin = sz->cx+4;
//		ymargin = sz->cy+4;
//	}

//	// Calculate our screen area
//	GetClientRect(hWnd, &rt);
	rt.left   = 0;
	rt.top    = 0;
	rt.right  = gDrawArea[0]->allocation.width;
	rt.bottom = gDrawArea[0]->allocation.height;
//	switch (graphscreen)
//	{
//	case 0: // Split into 4 equal parts

	g1.left   =                                    xmargin;
	g1.top    =                                    ymargin;
	g1.right  = gDrawArea[0]->allocation.width   - xmargin;
	g1.bottom = gDrawArea[0]->allocation.height  - ymargin;

	g2.left   =                                    xmargin;
	g2.top    =                                    ymargin;
	g2.right  = gDrawArea[1]->allocation.width   - xmargin;
	g2.bottom = gDrawArea[1]->allocation.height  - ymargin;

	f1.left   =                                    xmargin;
	f1.top    =                                    ymargin;
	f1.right  = gDrawArea[2]->allocation.width   - xmargin;
	f1.bottom = gDrawArea[2]->allocation.height  - ymargin;

	f2.left   =                                    xmargin;
	f2.top    =                                    ymargin;
	f2.right  = gDrawArea[3]->allocation.width   - xmargin;
	f2.bottom = gDrawArea[3]->allocation.height  - ymargin;

//		break;
//	case 1: // Top half
//		x1 = rt.left     + xmargin;
//		y1 = rt.top      + ymargin;
//		x2 = rt.right    - xmargin;
//		y2 = rt.bottom/2 - ymargin/2;
//		break;
//	case 2: // Bottom half
//		x1 = rt.left     + xmargin;
//		y1 = rt.bottom/2 + ymargin/2;
//		x2 = rt.right    - xmargin;
//		y2 = rt.bottom   - ymargin;
//		break;
//	}

	
	if (ch1data!=NULL) DrawOneGraph(gDrawArea[0], gPixmap[0], g1.left, g1.top, g1.right, g1.bottom, ch1data, size1, 0);
	if (ch2data!=NULL) DrawOneGraph(gDrawArea[1], gPixmap[1], g2.left, g2.top, g2.right, g2.bottom, ch2data, size2, 1);
	if (ch1data!=NULL) DrawOneGraph(gDrawArea[2], gPixmap[2], f1.left, f1.top, f1.right, f1.bottom, ch1data, size1, 2);
	if (ch2data!=NULL) DrawOneGraph(gDrawArea[3], gPixmap[3], f2.left, f2.top, f2.right, f2.bottom, ch2data, size2, 3);

	font      = gtk_style_get_font(gDrawArea[0]->style);
	if (first)
	{
	   // Create 3 colours
	   red.red      = 0xffff;
	   red.green    = 0;
	   red.blue     = 0;
	   gdk_color_parse("red"   , &red);
	   purple.red   = 192<<8;
	   purple.green = 0;
	   purple.blue  = 192<<8;
	   gdk_color_parse("purple", &purple);
	   blue.red     = 0xffff;
	   blue.green   = 0;
	   blue.blue    = 0;
	   gdk_color_parse("blue"  , &blue);
	   green.red     = 0;
	   green.green   = 0xffff;
	   green.blue    = 0;
	   gdk_color_parse("green"  , &green);
	   // Create a style using the 3 colours. We 'hijack' the text colours,
	   // as on the drawing area widget they are further not used - in this
	   // example / program.
	   rc_style = gtk_rc_style_new();
	   rc_style->text[GTK_STATE_NORMAL  ]            = red;
	   rc_style->text[GTK_STATE_ACTIVE  ]            = purple;
	   rc_style->text[GTK_STATE_SELECTED]            = blue;
	   rc_style->text[GTK_STATE_INSENSITIVE]         = green;
	   rc_style->color_flags[GTK_STATE_NORMAL  ]    |= GTK_RC_TEXT;
	   rc_style->color_flags[GTK_STATE_ACTIVE  ]    |= GTK_RC_TEXT;
	   rc_style->color_flags[GTK_STATE_SELECTED]    |= GTK_RC_TEXT;
	   rc_style->color_flags[GTK_STATE_INSENSITIVE] |= GTK_RC_TEXT;
	   // Modify the text colours of each drawing area.
	   gtk_widget_modify_style(gDrawArea[0], rc_style);
	   gtk_widget_modify_style(gDrawArea[1], rc_style);
	   gtk_widget_modify_style(gDrawArea[2], rc_style);
	   gtk_widget_modify_style(gDrawArea[3], rc_style);
	   // Dispose of style.
	   gtk_rc_style_unref(rc_style);
	   // Only need to do this once.
	   first=0;
	}
	
	gdk_draw_string(gPixmap[0], font, gDrawArea[0]->style->text_gc[GTK_STATE_ACTIVE], g1.left, 10/*rt.top-ymargin-2*/, "Data");
	gdk_draw_string(gPixmap[0], font, gDrawArea[0]->style->text_gc[GTK_STATE_NORMAL],
                        (int)((rt.right-rt.left)*0.50)-4*xmargin, rt.bottom-rt.top-1, "Channel 1");

	gdk_draw_string(gPixmap[1], font, gDrawArea[1]->style->text_gc[GTK_STATE_ACTIVE], g2.left, 10/*rt.top-ymargin-2*/, "Data");
	gdk_draw_string(gPixmap[1], font, gDrawArea[1]->style->text_gc[GTK_STATE_NORMAL],
                        (int)((rt.right-rt.left)*0.50)-4*xmargin, rt.bottom-rt.top-1, "Channel 2");

	gdk_draw_string(gPixmap[2], font, gDrawArea[2]->style->text_gc[GTK_STATE_ACTIVE], f1.left, 10/*rt.top-ymargin-2*/, "FT");

	gdk_draw_string(gPixmap[3], font, gDrawArea[2]->style->text_gc[GTK_STATE_ACTIVE], f2.left, 10/*rt.top-ymargin-2*/, "FT");

#if DDC
	/* This is the text with the FFTs. The actual data size will be half            */
	/* that of the input data size! This is where the extra factor of 2 comes from. */
	if (size2>(4*(f1.right-f1.left)))
	{
	}
	else
	if (size2>(2*(f1.right-f1.left)))
	{
	   gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                       f1.left         , f1.bottom+ymargin-2, "0");
#if IO5
	   gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                       f1.left+size2/4 , f1.bottom+ymargin-2, "52 Mhz");
#else
	   gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                       f1.left+size2/4 , f1.bottom+ymargin-2, "25 Mhz");
#endif
	   gdk_draw_string(gPixmap[3], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                       f2.left         , f1.bottom+ymargin-2, "0");
	   gdk_draw_string(gPixmap[3], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                       f2.left+size2/4 , f1.bottom+ymargin-2, "50 kHz");
	}
	else
	{
#if IO5
	   gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                             f1.left         , f1.bottom+ymargin-2, "0");
	   gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                             f1.left+size2/4 , f1.bottom+ymargin-2, "52 Mhz");
	   gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                             f1.left+size2/2 , f1.bottom+ymargin-2, "105 Mhz");
#else
	   gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                             f1.left         , f1.bottom+ymargin-2, "0");
	   gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                             f1.left+size2/4 , f1.bottom+ymargin-2, "25 Mhz");
	   gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                             f1.left+size2/2 , f1.bottom+ymargin-2, "50 Mhz");
#endif
	   gdk_draw_string(gPixmap[3], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                             f2.left         , f1.bottom+ymargin-2, "0");
	   gdk_draw_string(gPixmap[3], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                             f2.left+size2/4 , f1.bottom+ymargin-2, "50 kHz");
	   gdk_draw_string(gPixmap[3], font, gDrawArea[0]->style->text_gc[GTK_STATE_INSENSITIVE],
                                             f2.left+size2/2 , f1.bottom+ymargin-2, "100 kHz");
	}

        gdk_draw_string(gPixmap[2], font, gDrawArea[0]->style->text_gc[GTK_STATE_NORMAL], (int)(f1.left+xmargin*2), f1.bottom+ymargin-2, "RF Signal");
        gdk_draw_string(gPixmap[3], font, gDrawArea[0]->style->text_gc[GTK_STATE_NORMAL], (int)(f2.left+xmargin*2), f1.bottom+ymargin-2, "DDC Output");
#endif

#if 0
	if (GetTextExtentPoint32(hdc, "Channel 1", 9, sz)==0)
	{
		xmargin = 72;
		ymargin = 16;
	}
	else
	{
		xmargin = sz->cx;
		ymargin = sz->cy;
	}

	mycolor = RGB(192, 0, 192);
	SetTextColor(hdc, mycolor);
	TextOut(hdc, g1.left, g1.top-ymargin-2, "Data", 4);
	TextOut(hdc, g2.left, g2.top-ymargin-2, "Data", 4);
	TextOut(hdc, f1.left, f1.top-ymargin-2, "FT"  , 2);
	TextOut(hdc, f2.left, f2.top-ymargin-2, "FT"  , 2);

	mycolor = RGB(255, 0, 0);
	SetTextColor(hdc, mycolor);
	TextOut(hdc, (int)((rt.right-rt.left)*0.25)-xmargin/2, (rt.bottom-rt.top)/2-ymargin/2, "Channel 1", 9);
	TextOut(hdc, (int)((rt.right-rt.left)*0.75)-xmargin/2, (rt.bottom-rt.top)/2-ymargin/2, "Channel 2", 9);

	ReleaseDC(hWnd, hdc);
#endif
}

#else

#if DDC
void DrawAxes(HDC hdc, int x1, int x2, int y, int y1, int y2, int x, int size, int chan)
#else
void DrawAxes(HDC hdc, int x1, int x2, int y, int y1, int y2, int x)
#endif
{
	HPEN pen, old;
	COLORREF mycolor = RGB(0, 0, 255);

	pen = CreatePen(PS_SOLID, 1, mycolor);
	old = (HPEN)SelectObject(hdc, pen);

	MoveToEx(hdc, x1, y, NULL);
	LineTo  (hdc, x2, y);
	MoveToEx(hdc, x1, y1, NULL);
	LineTo  (hdc, x1, y2);

#if DDC
//	{
//		char msg[256];
//		sprintf(msg, "x2=%d,x1=%d,x2-x1=%d,size=%d", x2, x1, x2-x1, size);
//		MessageBox(NULL, msg, "", MB_OK);
//	}

	if ((chan==2)||(chan==3))
	{
		if (size > (2*(x2-x1)))
		{
		}
		else
		if (size > (x2-x1))
		{
			MoveToEx(hdc, x1+(size/2), y-4, NULL);
			LineTo  (hdc, x1+(size/2), y+4);
		}
		else
		{
			MoveToEx(hdc, x1+(size/2), y-4, NULL);
			LineTo  (hdc, x1+(size/2), y+4);
			MoveToEx(hdc, x1+ size   , y-4, NULL);
			LineTo  (hdc, x1+ size   , y+4);
		}
	}
#endif
	SelectObject(hdc, old);
	DeleteObject(pen);
}

void DrawOneGraph_si(HWND hWnd, HDC hdc, int x1, int y1, int x2, int y2, int *data, int size, int chan, int autoscale)
{
	if (autoscale)
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=data[0];
		min=data[0];
		for (i=1; i<size; i++)
		{
			if (data[i]>max) max=data[i];
			if (data[i]<min) min=data[i];
		}

		// Change all to a relative position ////////////////////////////////////////
		if (min>0) min = 0;
		if (max<0) max = 0;
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

		// Draw the graph ///////////////////////////////////////////////////////////
		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
		if (min<0)
		{
			y = ((float)max/((float)(max-min)))*s+y1;
		}
		else
		{
			y = y2;
		}
#if DDC
		DrawAxes(hdc, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(hdc, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = y;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		if (min<0)
		{
			y   = -(int)(f*(float)(data[0]-min)) + y2;
		}
		else
		{
			y   = -(int)(f*(float)data[0]) + y2;
		}
		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]    = size;
		olddata[chan][0] = y;
		oldv   [chan]    = v; 
		if (min<0)
		{
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)(data[i] - min)) + y2;
				LineTo  (hdc, v*i+x1, y);
				MoveToEx(hdc, v*i+x1, y, NULL);
				olddata[chan][i] = y;
			}
		}
		else
		{
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)data[i]) + y2;
				LineTo  (hdc, v*i+x1, y);
				MoveToEx(hdc, v*i+x1, y, NULL);
				olddata[chan][i] = y;
			}
		}
		ReleaseDC(hWnd, hdc);
	}
	else
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=ChGraphData[chan].max_y_value;		// use user specified maximum
		min=ChGraphData[chan].min_y_value;		// use user specified minimum
		if (max<min)
		{
			int tmp;
			tmp = max;
			max = min;
			min = tmp;
		}

		// Change all to a relative position ////////////////////////////////////////
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

		// Draw the graph ///////////////////////////////////////////////////////////
		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
		if (min<0)
		{
			y = -(int)(f*(float)(ChGraphData[chan].min_y_value-min)) + y2;
		}
		else
		{
			y = -(int)(f*(float)ChGraphData[chan].min_y_value) + y2;
		}
#if DDC
		DrawAxes(hdc, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(hdc, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		if (min<0)
		{
			y = -(int)(f*(float)(data[0]-min)) + y2;
		}
		else
		{
			y = -(int)(f*(float)data[0]) + y2;
		}
		if (y<y1) y=y1;
		else
		if (y>y2) y=y2;
		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]   =size;
		oldv   [chan]   =v;
		olddata[chan][0]=y;
		if (min<0)
		{
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)(data[i]-min)) + y2;
				if (y<y1) y=y1;
				else
				if (y>y2) y=y2;
				LineTo  (hdc, v*i+x1, y);
				MoveToEx(hdc, v*i+x1, y, NULL);
				olddata[chan][i]=y;
			}
		}
		else
		{
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)data[i]) + y2;
				if (y<y1) y=y1;
				else
				if (y>y2) y=y2;
				LineTo  (hdc, v*i+x1, y);
				MoveToEx(hdc, v*i+x1, y, NULL);
				olddata[chan][i]=y;
			}
		}
		ReleaseDC(hWnd, hdc);
	}
}

void DrawOneGraph_ui(HWND hWnd, HDC hdc, int x1, int y1, int x2, int y2, unsigned int *data, int size, int chan, int autoscale)
{
	if (autoscale)
	{
		unsigned int	max, min;
		int				i, y, d, s, v;
		float			f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=data[0];
		min=data[0];
		for (i=1; i<size; i++)
		{
			if (data[i]>max) max=data[i];
			if (data[i]<min) min=data[i];
		}

		// Change all to a relative position ////////////////////////////////////////
		if (min>0) min = 0;
		if (max<0) max = 0;
		d =  max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

		// Draw the graph ///////////////////////////////////////////////////////////
		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
#if DDC
		DrawAxes(hdc, x1, x2, y2, y1, y2, x1, size, chan);
#else
		DrawAxes(hdc, x1, x2, y2, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		y   = -(int)(f*(float)data[0]) + y2;
		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]    = size;
		oldv   [chan]    = v;
		olddata[chan][0] = y;
		for (i=1; (i<size)&&(i<(x2-x1)); i++)
		{
			y = -(int)(f*(float)data[i]) + y2;
			LineTo  (hdc, v*i+x1, y);
			MoveToEx(hdc, v*i+x1, y, NULL);
			olddata[chan][i] = y;
		}
		ReleaseDC(hWnd, hdc);
	}
	else
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=ChGraphData[chan].max_y_value;		// use user specified maximum
		min=ChGraphData[chan].min_y_value;		// use user specified minimum
		if (max<min)
		{
			int tmp;
			tmp = max;
			max = min;
			min = tmp;
		}

		// Change all to a relative position ////////////////////////////////////////
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

		// Draw the graph ///////////////////////////////////////////////////////////
		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
		if (max<0) y = y1;
		else
		if (min>0) y = y2;
		else       y = (int)(f*(float)ChGraphData[chan].min_y_value) + y2;
#if DDC
		DrawAxes(hdc, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(hdc, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		y   = -(int)(f*(float)data[0]-f*(float)ChGraphData[chan].min_y_value) + y2;
		if (y<y1) y=y1;
		else
		if (y>y2) y=y2;
		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]   =size;
		oldv   [chan]   =v;
		olddata[chan][0]=y;
		for (i=1; (i<size)&&(i<(x2-x1)); i++)
		{
			y = -(int)(f*(float)data[i]-f*(float)ChGraphData[chan].min_y_value) + y2;
			if (y<y1) y=y1;
			else
			if (y>y2) y=y2;
			LineTo  (hdc, v*i+x1, y);
			MoveToEx(hdc, v*i+x1, y, NULL);
			olddata[chan][i]=y;
		}
		ReleaseDC(hWnd, hdc);
	}
}

void DrawOneGraph_ss(HWND hWnd, HDC hdc, int x1, int y1, int x2, int y2, short int *data, int size, int chan, int autoscale)
{
	if (autoscale)
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=data[0];
		min=data[0];
		for (i=1; i<size; i++)
		{
			if (data[i]>max) max=data[i];
			if (data[i]<min) min=data[i];
		}

		// Change all to a relative position ////////////////////////////////////////
		if (min>0) min = 0;
		if (max<0) max = 0;
		d =  max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

		// Draw the graph ///////////////////////////////////////////////////////////
		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
		if (min<0)
		{
			y = ((float)max/((float)(max-min)))*s+y1;
		}
		else
		{
			y = y2;
		}
#if DDC
		DrawAxes(hdc, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(hdc, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = y;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		if (min<0)
		{
			y   = -(int)(f*(float)(data[0]-min)) + y2;
			MoveToEx(hdc, x1, y, NULL);
			oldsize[chan]    = size;
			oldv   [chan]    = v;
			olddata[chan][0] = y;
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)(data[i]-min)) + y2;
				LineTo  (hdc, v*i+x1, y);
				MoveToEx(hdc, v*i+x1, y, NULL);
				olddata[chan][i] = y;
			}
		}
		else
		{
			y   = -(int)(f*(float)data[0]) + y2;
			MoveToEx(hdc, x1, y, NULL);
			oldsize[chan]    = size;
			oldv   [chan]    = v;
			olddata[chan][0] = y;
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)data[i]) + y2;
				LineTo  (hdc, v*i+x1, y);
				MoveToEx(hdc, v*i+x1, y, NULL);
				olddata[chan][i] = y;
			}
		}
		ReleaseDC(hWnd, hdc);
	}
	else
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=ChGraphData[chan].max_y_value;		// use user specified maximum
		min=ChGraphData[chan].min_y_value;		// use user specified minimum
		if (max<min)
		{
			int tmp;
			tmp = max;
			max = min;
			min = tmp;
		}

		// Change all to a relative position ////////////////////////////////////////
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

		// Draw the graph ///////////////////////////////////////////////////////////
		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
#if DDC
		DrawAxes(hdc, x1, x2, y2, y1, y2, x1, size, chan);
#else
		DrawAxes(hdc, x1, x2, y2, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		if (min<0)
		{
			y   = -(int)(f*(float)(data[0]-min)) + y2;
			if (y<y1) y=y1;
			else
			if (y>y2) y=y2;
			MoveToEx(hdc, x1, y, NULL);
			oldsize[chan]   =size;
			oldv   [chan]   =v;
			olddata[chan][0]=y;
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)(data[i]-min)) + y2;
				if (y<y1) y=y1;
				else
				if (y>y2) y=y2;
				LineTo  (hdc, v*i+x1, y);
				MoveToEx(hdc, v*i+x1, y, NULL);
				olddata[chan][i]=y;
			}
		}
		else
		{
			y   = -(int)(f*(float)data[0]) + y2;
			if (y<y1) y=y1;
			else
			if (y>y2) y=y2;
			MoveToEx(hdc, x1, y, NULL);
			oldsize[chan]   =size;
			oldv   [chan]   =v;
			olddata[chan][0]=y;
			for (i=1; (i<size)&&(i<(x2-x1)); i++)
			{
				y = -(int)(f*(float)data[i]) + y2;
				if (y<y1) y=y1;
				else
				if (y>y2) y=y2;
				LineTo  (hdc, v*i+x1, y);
				MoveToEx(hdc, v*i+x1, y, NULL);
				olddata[chan][i]=y;
			}
		}
		ReleaseDC(hWnd, hdc);
	}
}

void DrawOneGraph_us(HWND hWnd, HDC hdc, int x1, int y1, int x2, int y2, unsigned short int *data, int size, int chan, int autoscale)
{
	if (autoscale)
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=data[0];
		min=data[0];
		for (i=1; i<size; i++)
		{
			if (data[i]>max) max=data[i];
			if (data[i]<min) min=data[i];
		}

		// Change all to a relative position ////////////////////////////////////////
		if (min>0) min = 0;
		if (max<0) max = 0;
		d =  max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

		// Draw the graph ///////////////////////////////////////////////////////////
		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
#if DDC
		DrawAxes(hdc, x1, x2, y2, y1, y2, x1, size, chan);
#else
		DrawAxes(hdc, x1, x2, y2, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		y   = -(int)(f*(float)data[0]) + y2;
		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]    = size;
		oldv   [chan]    = v;
		olddata[chan][0] = y;
		for (i=1; (i<size)&&(i<(x2-x1)); i++)
		{
			y = -(int)(f*(float)data[i]) + y2;
			LineTo  (hdc, v*i+x1, y);
			MoveToEx(hdc, v*i+x1, y, NULL);
			olddata[chan][i] = y;
		}
		ReleaseDC(hWnd, hdc);
	}
	else
	{
		int   max, min, i, y, d, s, v;
		float f;

		// Find top and bottom data value ///////////////////////////////////////////
		max=ChGraphData[chan].max_y_value;		// use user specified maximum
		min=ChGraphData[chan].min_y_value;		// use user specified minimum
		if (max<min)
		{
			int tmp;
			tmp = max;
			max = min;
			min = tmp;
		}

		// Change all to a relative position ////////////////////////////////////////
		d = max - min;				// space we need
		s = y2  - y1;				// space available
		if (d   ==0) f = 1.0;
		else         f = (float) s / (float) d;
		if (size==0) v = 1;
		else         v = (x2-x1)/size;
		if (!v) v=1;

		// Draw the graph ///////////////////////////////////////////////////////////
		hdc = GetDC(hWnd);
		// Draw the two axes ////////////////////////////////////////////////////////
		if (max<0) y = y1;
		else
		if (min>0) y = y2;
		else       y = (int)(f*(float)ChGraphData[chan].min_y_value) + y2;
#if DDC
		DrawAxes(hdc, x1, x2, y, y1, y2, x1, size, chan);
#else
		DrawAxes(hdc, x1, x2, y, y1, y2, x1);
#endif
		oldy[chan] = 0;

		// Draw the data ////////////////////////////////////////////////////////////
		// Calculate y position: scale (y'=f*y), then invert (y"=-y'+s); 
		// Hence y'=-(f*y)+s, but still need to add ymargin (y1).
		y   = -(int)(f*(float)data[0]-f*(float)ChGraphData[chan].min_y_value) + y2;
		if (y<y1) y=y1;
		else
		if (y>y2) y=y2;
		MoveToEx(hdc, x1, y, NULL);
		oldsize[chan]   =size;
		oldv   [chan]   =v;
		olddata[chan][0]=y;
		for (i=1; (i<size)&&(i<(x2-x1)); i++)
		{
			y = -(int)(f*(float)data[i]-f*(float)ChGraphData[chan].min_y_value) + y2;
			if (y<y1) y=y1;
			else
			if (y>y2) y=y2;
			LineTo  (hdc, v*i+x1, y);
			MoveToEx(hdc, v*i+x1, y, NULL);
			olddata[chan][i]=y;
		}
		ReleaseDC(hWnd, hdc);
	}
}

void DoDFT_si(int *chdata, int *fftdata, int size)
{
	int		i, max, min;
	float   f, data[1024];
	COMPLEX mydata[1024];

	if (size>1024)
	{
		MessageBox(NULL, "FFT size must be 1024 or lower.", "FFT", MB_OK);
		return;
	}

	// Normalise input data: to between -100 and 100
	max = chdata[0];
	min = chdata[0];
	for (i=1; i<size; i++)
	{
		if (chdata[i]>max) max=chdata[i];
		if (chdata[i]<min) min=chdata[i];
	}

	if (min<0)
	{
		if (max==min)
			f = 1.0;
		else
			f = (float) 100.0 / (float) (max-min);
	}
	else
	{
		if (max==0) 
			f = 1.0;
		else
			f = (float) 100.0 / (float) max;
	}

	for (i=0; i<size; i++)
	{
		data[i] = f * (float) (chdata[i]);
	}

	spdftr(data, mydata, size);
	for (i=0; i<size/2; i++)
	{
		fftdata[i] = (int) sqrt(mydata[i].real*mydata[i].real+mydata[i].imag*mydata[i].imag);
	}
}

void DoDFT_ui(unsigned int *chdata, unsigned int *fftdata, int size)
{
	int				i;
	unsigned int	max, min;
	float			f, data[1024];
	COMPLEX			mydata[1024];

	if (size>1024)
	{
		MessageBox(NULL, "FFT size must be 1024 or lower.", "FFT", MB_OK);
		return;
	}

	// Normalise input data: to between -100 and 100
	max = chdata[0];
	min = chdata[0];
	for (i=1; i<size; i++)
	{
		if (chdata[i]>max) max=chdata[i];
		if (chdata[i]<min) min=chdata[i];
	}

	if (min<0)
	{
		if (max==min)
			f = 1.0;
		else
			f = (float) 100.0 / (float) (max-min);
	}
	else
	{
		if (max==0) 
			f = 1.0;
		else
			f = (float) 100.0 / (float) max;
	}

	for (i=0; i<size; i++)
	{
		data[i] = f * (float) (chdata[i]);
	}

	spdftr(data, mydata, size);
	for (i=0; i<size/2; i++)
	{
		fftdata[i] = (int) sqrt(mydata[i].real*mydata[i].real+mydata[i].imag*mydata[i].imag);
	}
}

void DoDFT_ss(short *chdata, short *fftdata, int size)
{
	int		i, max, min;
	float   f, data[1024];
	COMPLEX mydata[1024];

	if (size>1024)
	{
		MessageBox(NULL, "FFT size must be 1024 or lower.", "FFT", MB_OK);
		return;
	}

	max = chdata[0];
	min = chdata[0];
	for (i=1; i<size; i++)
	{
		if (chdata[i]>max) max=chdata[i];
		if (chdata[i]<min) min=chdata[i];
	}

	if (min<0)
	{
		if (max==min)
			f = 1.0;
		else
			f = (float) 100.0 / (float) (max-min);
	}
	else
	{
		if (max==0) 
			f = 1.0;
		else
			f = (float) 100.0 / (float) max;
	}

	for (i=0; i<size; i++)
	{
		data[i] = f * (float) (chdata[i]);
	}

	spdftr(data, mydata, size);
	for (i=0; i<size/2; i++)
	{
		fftdata[i] = (int) sqrt(mydata[i].real*mydata[i].real+mydata[i].imag*mydata[i].imag);
	}
}

void DoDFT_us(unsigned short *chdata, unsigned short *fftdata, int size)
{
	int		i, max, min;
	float   f, data[1024];
	COMPLEX mydata[1024];

	if (size>1024)
	{
		MessageBox(NULL, "FFT size must be 1024 or lower.", "FFT", MB_OK);
		return;
	}

	// Normalise input data: to between -100 and 100
	max = chdata[0];
	min = chdata[0];
	for (i=1; i<size; i++)
	{
		if (chdata[i]>max) max=chdata[i];
		if (chdata[i]<min) min=chdata[i];
	}

	if (min<0)
	{
		if (max==min)
			f = 1.0;
		else
			f = (float) 100.0 / (float) (max-min);
	}
	else
	{
		if (max==0) 
			f = 1.0;
		else
			f = (float) 100.0 / (float) max;
	}

	for (i=0; i<size; i++)
	{
		data[i] = f * (float) (chdata[i]);
	}

	spdftr(data, mydata, size);
	for (i=0; i<size/2; i++)
	{
		fftdata[i] = (int) sqrt(mydata[i].real*mydata[i].real+mydata[i].imag*mydata[i].imag);
	}
}

void DoDFT(int datatype, void *chdata, void *fftdata, int size)
{
	switch (datatype)
	{
	case 0: DoDFT_si((           int *)chdata, (           int *)fftdata, size); break;
	case 1: DoDFT_ss((         short *)chdata, (         short *)fftdata, size); break;
	case 2: DoDFT_ui((unsigned   int *)chdata, (unsigned   int *)fftdata, size); break;
	case 3: DoDFT_us((unsigned short *)chdata, (unsigned short *)fftdata, size); break;
	}
}

void DrawOneGraph(HWND hWnd, HDC hdc, int x1, int y1, int x2, int y2, void *data, int size, int chan)
{
	static int autoscale, datatype, first = 1;
	int fftdata[1024];

	// Erase previous graph
	if (!first)
	{
		HPEN pen, old;
		int  i,v;

		pen = CreatePen(PS_SOLID, 1, GetBkColor(hdc));
		old = (HPEN)SelectObject(hdc, pen);

		// delete old x-axis - if necessary
		if (oldy[chan])
		{
			MoveToEx(hdc, x1, oldy[chan], NULL);
			LineTo  (hdc, x2, oldy[chan]);
		}

		// get old index mutiplier
		v   = oldv[chan];
		// move to first data point
		MoveToEx(hdc, x1, olddata[chan][0], NULL);
		for (i=1; i<oldsize[chan]; i++)
		{
			LineTo  (hdc, v*i+x1, olddata[chan][i]);
			MoveToEx(hdc, v*i+x1, olddata[chan][i], NULL);
		}
		SelectObject(hdc, old);
		DeleteObject(pen);
	}
	first=0;

	switch (chan)
	{
	case 0  : autoscale = ChGraphData[0].autoscale; datatype = ChGraphData[0].data_type; break;
	case 1  : autoscale = ChGraphData[1].autoscale; datatype = ChGraphData[1].data_type; break;
	case 2  : if (!dofft1) return;
			  autoscale = 1;                        datatype = ChGraphData[0].data_type;
			  DoDFT(datatype, data, fftdata, size); data = fftdata; size=size/2;
			  break;
	case 3  : if (!dofft2) return;
			  autoscale = 1;                        datatype = ChGraphData[1].data_type;
		      DoDFT(datatype, data, fftdata, size); data = fftdata; size=size/2;
			  break;
	default : return;
	}

	switch (datatype)
	{
	case 0: DrawOneGraph_si(hWnd, hdc, x1, y1, x2, y2, (           int *)data, size, chan, autoscale); break;
	case 1: DrawOneGraph_ss(hWnd, hdc, x1, y1, x2, y2, (         short *)data, size, chan, autoscale); break;
	case 2: DrawOneGraph_ui(hWnd, hdc, x1, y1, x2, y2, (unsigned   int *)data, size, chan, autoscale); break;
	case 3: DrawOneGraph_us(hWnd, hdc, x1, y1, x2, y2, (unsigned short *)data, size, chan, autoscale); break;
	}
}

void DrawGraph(HWND hWnd, HDC hdc, void *ch1data, void *ch2data, int size1, int size2)
{
	int i=0, x1, y1, x2, y2;
	RECT rt, g1, g2, f1, f2;
	SIZE Sz, *sz;
	COLORREF mycolor = RGB(192, 0, 192);

	sz = &Sz;

	hdc = GetDC(hWnd);
	if (GetTextExtentPoint32(hdc, "A", 1, sz)==0)
	{
		xmargin =  8;
		ymargin = 16;
	}
	else
	{
		char msg[256];
		sprintf(msg, "cx=%d, cy=%d", sz->cx, sz->cy);
//		MessageBox(NULL, msg, "sz", MB_OK);
		xmargin = sz->cx+4;
		ymargin = sz->cy+4;
	}

	// Calculate our screen area
	GetClientRect(hWnd, &rt);
	switch (graphscreen)
	{
	case 0: // Split into 4 equal parts
		g1.left   = rt.left     + xmargin;
		g1.top    = rt.top      + ymargin;
		g1.right  = rt.right/2  - xmargin;
		g1.bottom = rt.bottom/2 - ymargin;
		g2.left   = rt.right/2  + xmargin;
		g2.top    = rt.top      + ymargin;
		g2.right  = rt.right    - xmargin;
		g2.bottom = rt.bottom/2 - ymargin;
		f1.left   = rt.left     + xmargin;
		f1.top    = rt.bottom/2 + ymargin;
		f1.right  = rt.right/2  - xmargin;
		f1.bottom = rt.bottom   - ymargin;
		f2.left   = rt.right/2  + xmargin;
		f2.top    = rt.bottom/2 + ymargin;
		f2.right  = rt.right    - xmargin;
		f2.bottom = rt.bottom   - ymargin;
		break;
	case 1: // Top half
		x1 = rt.left     + xmargin;
		y1 = rt.top      + ymargin;
		x2 = rt.right    - xmargin;
		y2 = rt.bottom/2 - ymargin/2;
		break;
	case 2: // Bottom half
		x1 = rt.left     + xmargin;
		y1 = rt.bottom/2 + ymargin/2;
		x2 = rt.right    - xmargin;
		y2 = rt.bottom   - ymargin;
		break;
	}

	if (ch1data!=NULL) DrawOneGraph(hWnd, hdc, g1.left, g1.top, g1.right, g1.bottom, ch1data, size1, 0);
	if (ch2data!=NULL) DrawOneGraph(hWnd, hdc, g2.left, g2.top, g2.right, g2.bottom, ch2data, size2, 1);
	if (ch1data!=NULL) DrawOneGraph(hWnd, hdc, f1.left, f1.top, f1.right, f1.bottom, ch1data, size1, 2);
	if (ch2data!=NULL) DrawOneGraph(hWnd, hdc, f2.left, f2.top, f2.right, f2.bottom, ch2data, size2, 3);
#if DDC
	if (GetTextExtentPoint32(hdc, "RF Signal", 9, sz)==0)
#else
	if (GetTextExtentPoint32(hdc, "Channel 1", 9, sz)==0)
#endif
	{
		xmargin = 72;
		ymargin = 16;
	}
	else
	{
		xmargin = sz->cx;
		ymargin = sz->cy;
	}

	mycolor = RGB(192, 0, 192);
	SetTextColor(hdc, mycolor);
	TextOut(hdc, g1.left, g1.top-ymargin-2, "Data", 4);
	TextOut(hdc, g2.left, g2.top-ymargin-2, "Data", 4);
	TextOut(hdc, f1.left, f1.top-ymargin-2, "FT"  , 2);
	TextOut(hdc, f2.left, f2.top-ymargin-2, "FT"  , 2);

#if DDC
	/* This is the text with the FFTs. The actual data size will be half            */
	/* that of the input data size! This is where the extra factor of 2 comes from. */
	if (size2>(4*(f1.right-f1.left)))
	{
	}
	else
	if (size2>(2*(f1.right-f1.left)))
	{
		mycolor = RGB(0, 255, 0);
		SetTextColor(hdc, mycolor);
		TextOut(hdc, f1.left         , f1.bottom+1, "0", 1);
#if IO5
		TextOut(hdc, f1.left+size2/4 , f1.bottom+1, "52 Mhz", 6);
#else
		TextOut(hdc, f1.left+size2/4 , f1.bottom+1, "25 Mhz", 6);
#endif
		TextOut(hdc, f2.left         , f1.bottom+1, "0", 1);
		TextOut(hdc, f2.left+size2/4 , f1.bottom+1, "50 kHz", 6);
	}
	else
	{
		mycolor = RGB(0, 255, 0);
		SetTextColor(hdc, mycolor);
#if IO5
		TextOut(hdc, f1.left         , f1.bottom+1, "0", 1);
		TextOut(hdc, f1.left+size2/4 , f1.bottom+1, "52 Mhz", 6);
		TextOut(hdc, f1.left+size2/2 , f1.bottom+1, "105 Mhz", 6);
#else
		TextOut(hdc, f1.left         , f1.bottom+1, "0", 1);
		TextOut(hdc, f1.left+size2/4 , f1.bottom+1, "25 Mhz", 6);
		TextOut(hdc, f1.left+size2/2 , f1.bottom+1, "50 Mhz", 6);
#endif
		TextOut(hdc, f2.left         , f1.bottom+1, "0", 1);
		TextOut(hdc, f2.left+size2/4 , f1.bottom+1, "50 kHz", 6);
		TextOut(hdc, f2.left+size2/2 , f1.bottom+1, "100 kHz", 7);
	}
#endif

	mycolor = RGB(255, 0, 0);
	SetTextColor(hdc, mycolor);
	TextOut(hdc, (int)((rt.right-rt.left)*0.25)-xmargin/2, (rt.bottom-rt.top)/2-ymargin/2, "Channel 1", 9);
	TextOut(hdc, (int)((rt.right-rt.left)*0.75)-xmargin/2, (rt.bottom-rt.top)/2-ymargin/2, "Channel 2", 9);

#if DDC
	if (GetTextExtentPoint32(hdc, "A", 1, sz)==0)
	{
		xmargin =  8;
		ymargin = 16;
	}
	else
	{
		xmargin = sz->cx+4;
		ymargin = sz->cy+4;
	}
	mycolor = RGB(255, 0, 0);
	SetTextColor(hdc, mycolor);
	TextOut(hdc, (int)(f1.left+xmargin*2), f1.bottom+1, "RF Signal", 9);
	TextOut(hdc, (int)(f2.left+xmargin*2), f1.bottom+1, "DDC Output", 10);
#endif

	ReleaseDC(hWnd, hdc);

}
#endif
