-- ==============================================================
-- FILE : FFT.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of FFT_Example, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : FFT
--  Date        : 02/04/03
--  Author      :T. Hollis
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     T. Hollis    02-04-03    First Written

--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity FFT is
  port (
    RST           : in  std_logic;
    SCLK          : in  std_logic;
    FFT_IP_I      : in  std_logic_vector(15 downto 0);
    FFT_IP_Q      : in  std_logic_vector(15 downto 0);
    FFT_OP_I      : out std_logic_vector(15 downto 0);
    FFT_OP_Q      : out std_logic_vector(15 downto 0);
    SYNC          : out std_logic;
    WIN_SEL       : in std_logic;
    FWD_FFT       : in std_logic;
    LED0          : out std_logic; -- FFT Started
    LED1          : out std_logic; -- Core BUSY
    RD_MEM_ADDR   : out std_logic_vector(9 downto 0);
    DIO_OUT       :	out std_logic_vector(7 downto 0);
    DIO_EN        :	out std_logic_vector(7 downto 0)
  );
end FFT;


architecture RTL of FFT is

component vfft1024v2
   port(
     clk           :  in  std_logic;
     rs            :  in  std_logic;
     start         :  in  std_logic;
     ce            :  in  std_logic;
     scale_mode    :  in  std_logic;
     di_r          :  in  std_logic_vector(15 downto 0);
     di_i          :  in  std_logic_vector(15 downto 0);
     fwd_inv       :  in  std_logic;
     io_mode0      :  in  std_logic;
     io_mode1      :  in  std_logic;
     mwr           :  in  std_logic;
     mrd           :  in  std_logic;
     ovflo         :  out std_logic;
     result        :  out std_logic;
     mode_ce       :  out std_logic;
     done          :  out std_logic;
     edone         :  out std_logic;
     io            :  out std_logic;
     eio           :  out std_logic;
     bank          :  out std_logic;
     busy          :  out std_logic;
     wea           :  out std_logic;
     WEA_X         :  out std_logic;
     wea_y         :  out std_logic;
     web_x         :  out std_logic;
     web_y         :  out std_logic;
     ena_x         :  out std_logic;
     ena_y         :  out std_logic;
     index         :  out std_logic_vector(9 downto 0);
     addrr_x       :  out std_logic_vector(9 downto 0);
     addrr_y       :  out std_logic_vector(9 downto 0);
     addrw_x       :  out std_logic_vector(9 downto 0);
     addrw_y       :  out std_logic_vector(9 downto 0);
     xk_r          :  out std_logic_vector(15 downto 0);
     xk_i          :  out std_logic_vector(15 downto 0);
     yk_r          :  out std_logic_vector(15 downto 0);
     yk_i          :  out std_logic_vector(15 downto 0)
     );
end component;

component fftmem1024x16
    port (
      addra        : in  std_logic_VECTOR(9 downto 0);
      addrb        : in  std_logic_VECTOR(9 downto 0);
      clka         : in  std_logic;
      clkb         : in  std_logic;
      dina         : in  std_logic_VECTOR(15 downto 0);
      dinb         : in  std_logic_VECTOR(15 downto 0);
      doutb        : out std_logic_VECTOR(15 downto 0);
      ena          : in  std_logic;
      enb          : in  std_logic;
      wea          : in  std_logic;
      web          : in  std_logic
      );
end component;

component fftmemz1024x16
    port (
      addra        : in  std_logic_VECTOR(9 downto 0);
      addrb        : in  std_logic_VECTOR(9 downto 0);
      clka         : in  std_logic;
      clkb         : in  std_logic;
      dina         : in  std_logic_VECTOR(15 downto 0);
      doutb        : out std_logic_VECTOR(15 downto 0);
      wea          : in  std_logic
      );
end component;

component rom1024x16
    port (
	addr          : in  std_logic_VECTOR(9 downto 0);
	clk           : in  std_logic;
	dout          : out std_logic_VECTOR(15 downto 0)
	);
end component;

component mult16
     port (
      clk          : in  std_logic;
      a            : in  std_logic_VECTOR(15 downto 0);
      b            : in  std_logic_VECTOR(15 downto 0);
      q            : out std_logic_VECTOR(15 downto 0)
	 );
end component;

  type STATE_TYPE is (S1, S2, S3, S4, S5);
  attribute ENUM_ENCODING: STRING;
  attribute ENUM_ENCODING of STATE_TYPE: type is "00001 00010 00100 01000 10000";
  signal CS, NS: STATE_TYPE;


  signal RESULT    : std_logic;
  signal WEA_X     : std_logic;
  signal WEA_Y     : std_logic;
  signal WEB_X     : std_logic;
  signal WEB_Y     : std_logic;
  signal ENA_X     : std_logic;
  signal ENA_Y     : std_logic;
  signal BANK      : std_logic;
  signal BUSY      : std_logic;
  signal MODE_CE   : std_logic;
  signal SYNCO     : std_logic;

  signal YK_I      : std_logic_vector(15 downto 0);
  signal YK_Q      : std_logic_vector(15 downto 0);
  signal XK_I      : std_logic_vector(15 downto 0);
  signal XK_Q      : std_logic_vector(15 downto 0);
  signal XN_I      : std_logic_vector(15 downto 0);
  signal XN_Q      : std_logic_vector(15 downto 0);

  signal ADDRR_X   : std_logic_vector(9 downto 0);
  signal ADDRR_Y   : std_logic_vector(9 downto 0);
  signal ADDRW_X   : std_logic_vector(9 downto 0);
  signal ADDRW_Y   : std_logic_vector(9 downto 0);
  signal INDEX     : std_logic_vector(9 downto 0);

  signal DI_I      : std_logic_vector(15 downto 0);
  signal DI_Q      : std_logic_vector(15 downto 0);
  signal DOX_I     : std_logic_vector(15 downto 0);
  signal DOX_Q     : std_logic_vector(15 downto 0);
  signal DOY_I     : std_logic_vector(15 downto 0);
  signal DOY_Q     : std_logic_vector(15 downto 0);

  signal MWR       : std_logic;
  signal START     : std_logic;

  signal WR_ADDR_B  : std_logic_vector(9 downto 0);
  signal WR_ADDR_B1 : std_logic_vector(9 downto 0);
  signal WR_ADDR_B2 : std_logic_vector(9 downto 0);
  signal WR_ADDR_B3 : std_logic_vector(9 downto 0);
  signal WR_ADDR_B4 : std_logic_vector(9 downto 0);
  signal ADDR_B     : unsigned(9 downto 0);
  signal FULL_X       : std_logic;

  signal RD_ADDR_Z  : std_logic_vector(9 downto 0);
  signal ADDR_Z     : unsigned(9 downto 0);
  signal FULL_Z     : std_logic;

  signal ADDR_MUX_X : std_logic_vector(9 downto 0);
  signal ADDR_MUX_Y : std_logic_vector(9 downto 0);

  signal WINDOW_COE : std_logic_vector(15 downto 0);
  signal MULT_IP    : std_logic_vector(15 downto 0);

  signal Z_OP_I     : std_logic_vector(15 downto 0);
  signal Z_OP_Q     : std_logic_vector(15 downto 0);


begin

 -- WRITE address MEMORY BANKS
  process(SCLK, START, MODE_CE)
  begin
    if (START='1' OR MODE_CE='1') then
      ADDR_B <= (others=>'0');
      FULL_X <= '0';
    elsif (rising_edge(SCLK) AND (FULL_X = '0') ) then
      ADDR_B <= ADDR_B + 1;

        if ADDR_B = X"3FE" then
           FULL_X <= '1';
        else
           FULL_X <= '0';
        end if;

    end if;
  end process;
  WR_ADDR_B   <= std_logic_vector(ADDR_B);
  RD_MEM_ADDR <= WR_ADDR_B;

-- Delay the write address to align with mult O/P
    process(SCLK, RST)
  begin
    if RST='1'  then
      WR_ADDR_B1 <= (others=>'0');
      WR_ADDR_B2 <= (others=>'0');
      WR_ADDR_B3 <= (others=>'0');
      WR_ADDR_B4 <= (others=>'0');
    elsif rising_edge(SCLK) then
      WR_ADDR_B1 <= WR_ADDR_B;
      WR_ADDR_B2 <= WR_ADDR_B1;
      WR_ADDR_B3 <= WR_ADDR_B2;
      WR_ADDR_B4 <= WR_ADDR_B3;
    end if;
  end process;

  -- ADDRESS MULTIPLEXER FOR INPUT\WORKING MEMORY BANKS
  process (WEB_Y, ADDRR_X, WR_ADDR_B)
    begin
       case WEB_Y is
            when '1' =>
        ADDR_MUX_X <= ADDRR_X;
        ADDR_MUX_Y <= WR_ADDR_B4;
            when '0' =>
        ADDR_MUX_X <= WR_ADDR_B4;
        ADDR_MUX_Y <= ADDRR_Y;
            when others =>
        ADDR_MUX_X <= ADDRR_X;
        ADDR_MUX_Y <= ADDRR_Y;
        end case;
  end process;

 
-- CONTROL STATE MACHINE FOR START UP
SYNC_PROC: process (SCLK, RST)
    begin
   if (RST='1') then
            CS <= S1;
       elsif rising_edge(SCLK) then
            CS <= NS;
       end if;
    end process;
 
COMB_PROC: process (CS, WEB_X )
    begin
       case CS is
            when S1 =>
        MWR   <= '0';
        START <= '0';
        LED0  <= '1';
        NS    <= S2;

           when S2 => -- MWR PULSE
        MWR   <= '1';
        START <= '0';
        LED0  <= '1';
        NS    <= S3;

           when S3 => -- MEMORY BANK X LOAD
        MWR   <= '0';
        START <= '0';
        LED0  <= '1';
        if (WEB_X = '1') then NS <= S3;
        else                  NS <= S4;
        end if;

           when S4 => -- START PULSE
        MWR   <= '0';
        START <= '1';
        LED0  <= '1';
        NS    <= S5;

           when S5 => -- START COMPLETE
        MWR   <= '0';
        START <= '0';
        LED0  <= '0';
        NS    <= S5;

      end case;
 end process;
 
  -- LED1 'ON' WHEN CORE IS PROCESSING DATA
  LED1 <= not BUSY;

  -- MEMORY BANK MULTIPLEXER(PHASE)
  process (BANK, DOX_I, DOY_I)
    begin
       case BANK is
            when '0' =>
        DI_I <= DOY_I;
            when '1' =>
        DI_I <= DOX_I;
            when others =>
        DI_I <= DOY_I;
       end case;
  end process;

  -- MEMORY BANK MULTIPLEXER(QUAD)
  process (BANK, DOX_Q, DOY_Q)
    begin
       case BANK is
            when '0' =>
        DI_Q <= DOY_Q;
            when '1' =>
        DI_Q <= DOX_Q;
            when others =>
        DI_Q <= DOY_Q;
       end case;
  end process;

 
 iTMS_FFT : vfft1024v2
     port map(
       clk        => SCLK,
       rs         => RST,
       start      => START,
       ce         => '1', --CLOCK ENABLE
       scale_mode => '0', --SCALE MODE 1/1024
       di_r       => DI_I,
       di_i       => DI_Q,
       fwd_inv     => FWD_FFT, --SET FOR FORWARD FFT

 --   Set for TMS
       io_mode0   => '1',
       io_mode1   => '0',

       mwr        => MWR,
       mrd        => '0', -- TIED LOW FOR TMS
       ovflo      => open,
       result     => RESULT,
       mode_ce    => MODE_CE,
       done       => SYNCO,
       edone      => open,
       io         => open,
       eio        => open,
       bank       => BANK,
       busy       => BUSY,
       wea        => open,
       wea_x      => WEA_X,
       wea_y      => WEA_Y,
       web_x      => WEB_X,
       web_y      => WEB_Y,
       ena_x      => ENA_X,
       ena_y      => ENA_Y,
       index      => INDEX,
       addrr_x    => ADDRR_X,
       addrr_y    => ADDRR_Y,
       addrw_x    => ADDRW_X,
       addrw_y    => ADDRW_Y,
       xk_r       => XK_I,
       xk_i       => XK_Q,
       yk_r       => YK_I,
       yk_i       => YK_Q
      );

 -- MEMORY BANK X
  iRAMX_I : fftmem1024x16
     port map (
       addra      => ADDRW_X,
       addrb      => ADDR_MUX_X,
       clka       => SCLK,
       clkb       => SCLK,
       dina       => XK_I,
       dinb       => XN_I,
       doutb      => DOX_I,
       ena        => ENA_X,
       enb        => '1',
       wea        => WEA_X,
       web        => WEB_X
      );

  iRAMX_Q : fftmem1024x16
     port map (
       addra      => ADDRW_X,
       addrb      => ADDR_MUX_X,
       clka       => SCLK,
       clkb       => SCLK,
       dina       => XK_Q,
       dinb       => XN_Q,
       doutb      => DOX_Q,
       ena        => ENA_X,
       enb        => '1',
       wea        => WEA_X,
       web        => WEB_X
      );

 -- MEMORY BANK Y
  iRAMY_I : fftmem1024x16
     port map (
       addra      => ADDRW_Y,
       addrb      => ADDR_MUX_Y,
       clka       => SCLK,
       clkb       => SCLK,
       dina       => XK_I,
       dinb       => XN_I,
       doutb      => DOY_I,
       ena        => ENA_Y,
       enb        => '1',
       wea        => WEA_Y,
       web        => WEB_Y
      );

 iRAMY_Q : fftmem1024x16
     port map (
       addra      => ADDRW_Y,
       addrb      => ADDR_MUX_Y,
       clka       => SCLK,
       clkb       => SCLK,
       dina       => XK_Q,
       dinb       => XN_Q,
       doutb      => DOY_Q,
       ena        => ENA_Y,
       enb        => '1',
       wea        => WEA_Y,
       web        => WEB_Y
      );

 -- READ address MEMORY BANK Z
  process(SCLK, RESULT)
  begin
    if RESULT ='1' then
      ADDR_Z <= (others=>'0');
      FULL_Z <= '0';
    elsif (rising_edge(SCLK) AND (FULL_Z = '0') ) then
      ADDR_Z <= ADDR_Z + 1;

        if ADDR_Z = X"3FE" then
           FULL_Z <= '1';
        else
           FULL_Z <= '0';		 
        end if;

    end if;
  end process;
  RD_ADDR_Z <= std_logic_vector(ADDR_Z);

 -- MEMORY BANK Z
  iRAMZ_I : fftmemz1024x16
     port map (
       addra      => INDEX,
       addrb      => RD_ADDR_Z,
       clka       => SCLK,
       clkb       => SCLK,
       dina       => YK_I,
       doutb      => Z_OP_I,
       wea        => RESULT
      );

 iRAMZ_Q : fftmemz1024x16
     port map (
       addra      => INDEX,
       addrb      => RD_ADDR_Z,
       clka       => SCLK,
       clkb       => SCLK,
       dina       => YK_Q,
       doutb      => Z_OP_Q,
       wea        => RESULT
      );

 --GAIN REQUIRED FOR INVERSE TRANSFORM
  process (FWD_FFT, Z_OP_I, Z_OP_Q)
    begin
       case FWD_FFT is
            when '0' =>
        FFT_OP_I(15)          <= Z_OP_I(15);
        FFT_OP_I(14 downto 0) <= Z_OP_I(8 downto 0) & "000000";
        FFT_OP_Q(15)          <= Z_OP_Q(15);
        FFT_OP_Q(14 downto 0) <= Z_OP_Q(8 downto 0) & "000000";
            when '1' =>
        FFT_OP_I <= Z_OP_I;
        FFT_OP_Q <= Z_OP_Q;
            when others =>
        FFT_OP_I <= Z_OP_I;
        FFT_OP_Q <= Z_OP_Q;
       end case;
  end process;

 iROM : rom1024x16
     port map (
       addr       => WR_ADDR_B,
       clk        => SCLK,
       dout       => WINDOW_COE
	  );


 -- SYNC PULSE FOR FFT O/P ON DAC'S
  SYNC         <= SYNCO;
  DIO_OUT(0)   <= SYNCO;
  DIO_EN(0)    <= '0';
  DIO_OUT(7 downto 1)<= (others=>'0');
  DIO_EN(7 downto 1) <= (others=>'1');


 ------------WINDOW-----------------
 -- Select Rectangular or K-B window
  process (WIN_SEL, WINDOW_COE)
    begin
       case WIN_SEL is
            when '0' =>
        MULT_IP <= WINDOW_COE;-- K-B
            when '1' =>
        MULT_IP <= X"ffff";-- RECTANGULAR
            when others =>
        MULT_IP <= WINDOW_COE;
       end case;
  end process;

 iMULT16_I : mult16
     port map (
       clk        => SCLK,
       a          => FFT_IP_I,
       b          => MULT_IP,
       q          => XN_I
	  );

  iMULT16_Q : mult16
     port map (
       clk        => SCLK,
       a          => FFT_IP_Q,
       b          => MULT_IP,
       q          => XN_Q
    	  );

 
end RTL;
