-- ==============================================================
-- FILE : FFT.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of FFT Example, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : FFT
--  Date        : 02/04/03
--  Author      :T. Hollis
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     T. Hollis    02-04-03    First Written

--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity FFT is
  port (
    RST           : in  std_logic;
    SCLK          : in  std_logic;
    FFTCLK          : in  std_logic;
    ADC_A         : in  std_logic_vector(11 downto 0); -- ADC_A data
  );
end FFT;


architecture RTL of FFT is



component bufmem1024x16
    port (
      addra        : in std_logic_VECTOR(9 downto 0);
      addrb        : in std_logic_VECTOR(9 downto 0);
      clka         : in std_logic;
      clkb         : in std_logic;
      dina         : in std_logic_VECTOR(15 downto 0);
      doutb        : out std_logic_VECTOR(15 downto 0);
      wea          : in std_logic
      );
end component;



  type STATE_TYPEW is (WS1, WS2, WS3, WS4, WS5, WS6);
  attribute ENUM_ENCODING: STRING;
  attribute ENUM_ENCODING of STATE_TYPEW: type is "000001 000010 000100 001000 010000 100000";
  signal CSW, NSW: STATE_TYPEW;

  type STATE_TYPER is (RS1, RS2, RS3, RS4, RS5, RS6);
  attribute ENUM_ENCODING of STATE_TYPER: type is "000000001 000000010 000000100 000001000 000010000 000100000 001000000 010000000 100000000";
  signal CSR, NSR: STATE_TYPER;


  signal R_COUNTER : unsigned(11 downto 0);  
  signal W_COUNTER : unsigned(9 downto 0);
  signal RD_ADDR   : std_logic_vector(11 downto 0);
  signal WR_ADDR   : std_logic_vector(9 downto 0);

begin


-- WRITE CONTROL STATE MACHINE
SYNC_PROC: process (SCLK, RST)
    begin
   if (RST='1') then
            CSW <= S1;
       elsif rising_edge(SCLK) then
            CSW <= NSW;
       end if;
    end process;
 
COMB_PROC: process (CSW, FULL, READ_COMP)
    begin
       case CSW is
            when WS1 =>
        WRCOUNT_RST <= '1';
        WRITE_EN_X  <= '1';
        WRITE_EN_Y  <= '0';
            NS <= WS2;

           when WS2 =>
        WRCOUNT_RST <= '0';
        WRITE_EN_X  <= '1';
        WRITE_EN_Y  <= '0';
            if   (FULL = '0') then NS <= WS2;
            else                   NS <= WS3;
            end if;

           when WS3 =>
        WRCOUNT_RST <= '0';
        WRITE_EN_X  <= '0';
        WRITE_EN_Y  <= '0';
            if (READ_COMP = '1') then NS <= WS4;
            else                      NS <= WS3;
            end if;

           when WS4 =>
        WRCOUNT_RST <= '1';
        WRITE_EN_X  <= '0';
        WRITE_EN_Y  <= '1';
        NS <= WS5;

          when WS5 =>
        WRCOUNT_RST <= '0';
        WRITE_EN_X  <= '0';
        WRITE_EN_Y  <= '1';
            if   (FULL = '0') then NS <= WS5;
            else                   NS <= WS6;
            end if;

          when WS6 =>
        WRCOUNT_RST <= '0';
        WRITE_EN_X  <= '0';
        WRITE_EN_Y  <= '0';
            if (READ_COMP = '1') then NS <= WS7;
            else                      NS <= WS1 ;
            end if;
    
       end case;
    end process;


-- READ CONTROL STATE MACHINE
    SYNC_PROC: process (FFTCLK, RST)
    begin
   if (RST='1') then
            CSR <= RS1;
       elsif rising_edge(FFTCLK) then
            CSR <= NSR;
       end if;
    end process;
 
COMB_PROC: process (CSR, EMPTY, WRITE_EN_X, WRITE_EN_Y)
    begin
       case CSR is
            when RS1 =>
        RDCOUNT_RST <= '1';
        READ_EN_X   <= '0';
        READ_EN_Y   <= '0';
        MEMW        <= '0';
        START       <= '0';
            NS1 <= RS2;

           when RS2 =>
        RDCOUNT_RST <= '1';
        READ_EN_X   <= '1';
        READ_EN_Y   <= '0';
        MEMW        <= '0';
        START       <= '0';
            if   (WRITE_EN_X = '1') then NS1 <= RS2;
            else                         NS1 <= RS3;
            end if;

           when RS3 =>
        RDCOUNT_RST <= '0';
        READ_EN_X   <= '1';
        READ_EN_Y   <= '0';
        MEMW        <= '1';
        START       <= '0';
        NS1 <= S4;

           when RS4 =>
        RDCOUNT_RST <= '0';
        READ_EN_X   <= '1';
        READ_EN_Y   <= '0';
        MEMW        <= '0';
        START       <= '0';
            if (EMPTY = '0')     then NS1 <= RS4;
            else                      NS1 <= RS5;
            end if;

          when RS5 =>
        RDCOUNT_RST <= '1';
        READ_EN_X   <= '0';
        READ_EN_Y   <= '1';
        MEMW        <= '0';
        START       <= '1';
            if (WRITE_EN_Y = '1') then NS1 <= RS5;
            else                       NS1 <= RS6;
            end if;

          when RS6 =>
        RDCOUNT_RST <= '0';
        READ_EN_X   <= '0';
        READ_EN_Y   <= '0';
        START       <= '0';
            if (EMPTY = '0')     then NS1 <= S6;
            else                      NS1 <= S7 ;
            end if;

          when RS7 =>
        RDCOUNT_RST <= '1';
        READ_EN_X   <= '1';
        READ_EN_Y   <= '0';
        MEMW        <= '0';
        START       <= '0';
            if (WRITE_EN_X = '1') then NS1 <= RS7;
            else                       NS1 <= RS8;
            end if;

          when RS8 =>
        RDCOUNT_RST <= '0';
        READ_EN_X   <= '1';
        READ_EN_Y   <= '0';
        MEMW        <= '0';
        START       <= '0';
            if (EMPTY = '0')     then NS1 <= RS8;
            else                      NS1 <= RS9;
            end if;

          when RS9 =>
        RDCOUNT_RST <= '1';
        READ_EN_X   <= '0';
        READ_EN_Y   <= '1';
        MEMW        <= '0';
        START       <= '0';
            if (WRITE_EN_Y = '1') then NS1 <= RS9;
            else                       NS1 <= RS6;
            end if;
    
       end case;
    end process;

process(SCLK)
begin
  if(rising_edge(SCLK))then





 -- READ address
  process(RDCOUNT_RST, FFTCLK)
  begin
    if RDCOUNT_RST='1' then
      RDcounter <= (others=>'0');
    elsif (rising_edge(FFTCLK) AND (NOT EMPTY) ) then
      RDcounter <= RDcounter + 1;

        if RDcounter = X"FFF" then
           EMPTY <= '1';
        else
           EMPTY <= '0';
        end if;

        if RDcounter(1 downto 0) = "11" then
           READ_EN <= '1';
        else
           READ_EN <= '0';
        end if;

    end if;
  end process;
  RD_ADDR <= std_logic_vector(R_COUNTER(11 downto 2) );
 
 -- WRITE address
  process(RST, SCLK, WRITE_EN)
  begin
    if WRCOUNT_RST='1' then
      W_COUNTER <= (others=>'0');
    elsif (rising_edge(SCLK) AND ((WRITE_EN_X='1') OR (WRITE_EN_Y='1'))) then
      W_COUNTER <= W_COUNTER + 1;

        if W_COUNTER = X"3FE" then
           FULL <= '1';
        else
           FULL <= '0';
        end if;

    end if;
  end process;
  WR_ADDR <= std_logic_vector(W_COUNTER);


iBUFX_I : bufmem1024x16
    port map (
     addra      => WR_ADDR,
     addrb      => RD_ADDR,
     clka       => SCLK,
     clkb       => FFTCLK,
     dina       => (ADC_A & "0000"),
     doutb      => DOX_I,
     wea        => WRITE_EN_X
    );

iBUFX_Q : bufmem1024x16
    port map (
     addra      => WR_ADDR,
     addrb      => RD_ADDR,
     clka       => SCLK,
     clkb       => FFTCLK,
     dina       => (ADC_A & "0000"),
     doutb      => DOX_Q,
     wea        => WRITE_EN_X
    );


iBUFY_I : bufmem1024x16
    port map (
     addra      => WR_ADDR,
     addrb      => RD_ADDR,
     clka       => SCLK,
     clkb       => FFTCLK,
     dina       => (ADC_A & "0000"),
     doutb      => DOY_I,
     wea        => WRITE_EN_Y
    );

iBUFY_Q : bufmem1024x16
    port map (
     addra      => WR_ADDR,
     addrb      => RD_ADDR,
     clka       => SCLK,
     clkb       => FFTCLK,
     dina       => (ADC_A & "0000"),
     doutb      => DOY_Q,
     wea        => WRITE_EN_Y
    );



end RTL;
