/************************************************************************/
/*                                                                      */
/* example2.c                                                           */
/*                                                                      */
/* Host based example program for demonstrating the example2 bitstream  */
/* for the HERON-IO5V.                                                  */
/*                                                                      */
/* 2001 Hunt Engineering                                                */
/* All Rights Reserved                                                  */
/*                                                                      */
/*  Ver Date        Author Comments                                     */
/*  --- ----        ------ --------                                     */
/*                                                                      */
/*  1.0 01/10/01    PW     first written                                */
/*  1.1 02/10/01    PW     ported to windows program                    */
/*  1.2 21/02/02    PW     added support for HEART (PC9)                */
/*  1.3 04/05/04    TH     Adapted for IO5V                             */
/*  2.0 06/03/03    TH     adapted for DDC                              */
/*  2.1 02/08/05    JT     added support for HEBASE2                    */
/*  3.0 04/12/06    JT     added support for LINUX                      */
/*                         Unified HeGraph source code                  */
/*                                                                      */
/************************************************************************/

#if _LINUX
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <semaphore.h>
#include <errno.h>
#include <math.h>
#else
#include "stdafx.h"
#include "resource.h"
#include "commdlg.h"
#include "stdio.h"
#include "math.h"
#include "process.h"
#include "errno.h"
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#endif

#include "heapi.h"
#include "settings.h"

#if _LINUX

void DrawWindow(int points, int *x);

#define HWND
#define hWnd

int stricmp(const char *s1, const char *s2)
{
   int i=0;
   while (s1[i])
   {
      if (tolower(s1[i])!=tolower(s2[i])) return (s1[i]-s2[i]);
      i++;
   }
   return 0;
}

extern int data_free;
extern int data_display;

#endif

HE_HANDLE 		hDevice;
HE_DWORD 		Status;


HE_WORD Board=0; // The board number we will be using
int verbose;
char devname[255]; // The Host Interface Type we will be using

/* Message Type Definitions */
#define MODULE_TYPE_QUERY	1
#define MODULE_TYPE_REPLY	2
#define SEND_BITSTREAM		3
#define USER_WRITE			8
#define USER_READ			9
#define USER_READ_REPLY		10

#define AM_INDEX 0.5
#define FM_INDEX 1.0

#define PI 3.1415926535         /* So we can calculate angles in radians */  

#if FFT
#define LUT_ADDR_BITS  10        /* Number of bits for DDS LUT */ 
#define TOT_ACC_BITS  26        /* Number of bits for DDS LUT */
#define SAMPLE_FREQ    100      /* Sample frequency in MHz */ 
#endif

/* Function Prototypes */
int   Initialise_Host_HSB(char *dev_name, HE_WORD board);
void error(char *string, int Status);
void SendUserData(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data);
void SendUserDatanr(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data);
int readback_user(HE_BYTE slot, HE_BYTE *reply, int buffer_size, HE_BYTE send_data);
void function1(HWND hWnd);
void function4(HWND hWnd);


/* Global Variables */
HE_HANDLE   hDevice			= NULL;
HE_IOSTATUS ReadIoStatus	= NULL;
HE_IOSTATUS WriteIoStatus	= NULL;
HE_HANDLE   hsbDevice		= NULL;
#define ADC_BLOCK 500
#define DAC_BLOCK 512
int slotnumber=1;
#if FFT
float ddsfreq = 1.5;
int fftwindow = 0;
int fft_reg = 8;
#endif

short data1[ADC_BLOCK];
short data2[ADC_BLOCK];
HE_DWORD chadata[DAC_BLOCK];
HE_DWORD chbdata[DAC_BLOCK];
int controlsem;
int ddc_reg;
int stopthread  = 1;

struct Params
{
	void *buf1;
	void *buf2;
	int   bufsize1;
	int   bufsize2;
};

struct Params params = { data1, data2, ADC_BLOCK, ADC_BLOCK };


#if _LINUX

char *getregenv(const char *varname)
{
   FILE         *f;
   char          fname[1024];
   static char   fbuf[1024];

   sprintf(fname, "/usr/local/bin/%s.dat", varname);
   f = fopen(fname, "r");
   if (f)
   {
      fgets (fbuf, 1024, f);
      fclose(f);
      return fbuf;
   }
   return NULL;
}

char *getapivar(const char *varname)
{
   return getregenv(varname);
}
#else

/************************************************************/
/*                                                          */
/*    GetWindowsVersion                                     */
/*                                                          */
/************************************************************/

#define OSUNK	0
#define OSW31	1
#define OSW9x	2
#define OSNTx	3

void GetWindowsVersion(int *type, int *ver)
{
	OSVERSIONINFO osvi;

	ZeroMemory(&osvi, sizeof(OSVERSIONINFO));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (GetVersionEx((OSVERSIONINFO *)&osvi))
	{
		if (osvi.dwPlatformId==VER_PLATFORM_WIN32_NT)
		{
			*type = OSNTx;
			*ver  = osvi.dwMajorVersion;
		}
		else
		if (osvi.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS)
		{
			*type = OSW9x;
			*ver  = osvi.dwMinorVersion;
		}
		else
		{
			*type = OSW31;
			*ver  = osvi.dwMajorVersion;
		}
	}
	else
	{
		*type = OSUNK;
		*ver  = osvi.dwMajorVersion;
	}
}

/************************************************************/
/*                                                          */
/*    getregenv                                             */
/*                                                          */
/* This function will first read a registry entry to find   */
/* a HUNT environmental variable. If the variable is not    */
/* found, getregenv will use 'getenv' to try to find the    */
/* environmantal variable. Reason is that NT/W2K/XP don't   */
/* seem to forward changes in environmental variables       */
/* unless the PC is rebooted. This way we can read the      */
/* latest change as done by confchk, but if a user only     */
/* uses system environmental variables it'll also work.     */
/*                                                          */
/************************************************************/

char *getregenv(const char *varname)
{
	static unsigned char kbuf[1024];
	DWORD    n=1024;
	int    type, ver;
	DWORD  ktype;

	GetWindowsVersion(&type, &ver);
	if (type==OSNTx)
	{
		char *envkey="SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment";
		HKEY  hkey;
		kbuf[0]=0;
		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, envkey, 0, KEY_QUERY_VALUE, &hkey)==ERROR_SUCCESS)
		{
			if (RegQueryValueEx(hkey, varname, NULL, &ktype, kbuf, &n)==ERROR_SUCCESS)
			{
				if ((ktype==REG_SZ)&&(n>0))
				{
					RegCloseKey(hkey);
					return (char *) kbuf;
				}
			}
			RegCloseKey(hkey);
		}
	}
	return getenv(varname);
}

/************************************************************/
/*                                                          */
/* getapivar                                                */
/*                                                          */
/************************************************************/

char *getapivar(const char *varname)
{
	HKEY                 huntkey;
	DWORD                ktype, n=1024;
	static unsigned char huntengstr[1024];
	char                *str, *huntkeystr;

	// First, look in the registry if our key is defined there.
	huntkeystr="Software\\HUNT ENGINEERING";
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, huntkeystr, 0, KEY_QUERY_VALUE, &huntkey)==ERROR_SUCCESS)
	{
		if (RegQueryValueEx(huntkey, varname, NULL, &ktype, huntengstr, &n)==ERROR_SUCCESS)
		{
			if ((ktype==REG_SZ)&&(n>0))
			{
				RegCloseKey(huntkey);
				return (char *) huntengstr;
			}
		}
		RegCloseKey(huntkey);
	}

	// OK, no such registry entry exists. Let's see if we have an environment
	// variable.
	return getregenv(varname);
}

#endif

/************************************************************/
/*                                                          */
/*    function0 -- run from file->start menu                */
/*                                                          */
/*  This loops around gathering buffers of data from the    */
/*  HERON-IO2 module. The speed of the loops is determined  */
/*  by how long the PC takes to run this program and draw   */
/*  the graph. Example2 for the HERON-IO2 captures the      */
/*  buffer when asked to do so by this program sending an   */
/*  HSB message.                                            */
/*  On each loop the semaphore is checked to see if the DAC */
/*  RAMS in the FPGA need to be updated                     */
/*                                                          */
/************************************************************/       


void function0(HWND hWnd)
{
	char		 tmp[255]; 
	char		*fifodev; 
	char		 command[255]; 
	int			 i,ask;
	int			 fifo;
	char		*string; 
	int			 start, length;
	int			 fifo_in,fifo_out;
#if DDC
	float		 phi_c_am,phi_c_fm,phi_m_am,phi_m_fm;
#endif
#if FFT
	double		 treal,timag,trealsq,timagsq,sum_sqs;
#endif
	FILE		*fptr;
	HE_DWORD	 info[21];
	HE_DWORD	 results[ADC_BLOCK]; /* DWORD as there are 2 channels */
		
	/* Set up for board we're going to open */
	string = NULL;
	string=getapivar("HEAPIHSB");
	if (string==NULL)
	{	
		string=getapivar("HEAPIJTAG");
		if (string==NULL)
		{
			error("No HEAPIHSB/JTAG variable found",-1);
			exit(-1);
		}
	}

	/* strip out the device name and the board number */
	i = 0;
	while (((string[i]==' ') || (string[i]=='\t'))&&(string[i]!=0)) i++;
	start=i;
	while (((string[i]!=' ') && (string[i]!='\t'))&&(string[i]!=0)) i++;
	length=i-start; 
	strncpy(devname,&string[start] , length);
	i++;
	while (((string[i]==' ') || (string[i]=='\t'))&&(string[i]!=0)) i++;
	start=i;
	while (((string[i]!=' ') && (string[i]!='\t'))&&(string[i]!=0)) i++;
	length=i=start; 
	strncpy(tmp,&string[start] , length);
	Board   = (HE_WORD)atoi(tmp);

	if ((Board>15)) error("Bad Board Number",-1);

	/* open the board and reset it*/
	Status = HeOpen1(devname, (HE_WORD)Board, "FifoA", &hDevice);
	if(Status != HE_OK) error("Open Failed, exiting\n", Status);

	/* reset the hardware*/
	HeReset(hDevice);

	/* Set up the FPGA for the demo */	
	// 0x40 = FIFO6
	// 0x20 = FIFO5
	// 0x10 = FIFO4
	// 0x08 = FIFO3
	// 0x04 = FIFO2
	// 0x02 = FIFO1
	// 0x01 = FIFO0

	if(stricmp(devname,"hep8a")==0)
	{
		fifo=1;
		fifodev="FifoA";
		slotnumber=1;
	}
	else
	if(stricmp(devname,"heb2a")==0)
	{
		switch (slotnumber)
		{
			case 2 : fifodev="FifoB"; fifo=1; break;
			default: fifodev="FifoA"; fifo=0; break;
		}
		if (fifodev!="FifoA")
		{
			Status = HeClose(&hDevice);
			if(Status != HE_OK) error("Failed close..., exiting\n", Status);
			Status = HeOpen1(devname, (HE_WORD)Board, fifodev, &hDevice);
			if(Status != HE_OK) error("Open Failed, exiting\n", Status);
		}
	}
	else
	{
		switch (slotnumber)
		{
			case 2 : fifodev="FifoB"; break;
			case 3 : fifodev="FifoC"; break;
			case 4 : fifodev="FifoD"; break;
			default: fifodev="FifoA"; break;
		}
		// The network file supplied with HeGraph connects 
		// always to fifo 0 on the module.
		fifo=0;
		if (fifodev!="FifoA")
		{
			Status = HeClose(&hDevice);
			if(Status != HE_OK) error("Failed close..., exiting\n", Status);
			Status = HeOpen1(devname, (HE_WORD)Board, fifodev, &hDevice);
			if(Status != HE_OK) error("Open Failed, exiting\n", Status);
		}
	}

	ReadIoStatus=NULL;
	WriteIoStatus=NULL;
	Status = HeInitIoStatus(hDevice, &ReadIoStatus); // Initialise read IoStatus
	if(Status != HE_OK) error("Can't init Read I/O, exiting\n", Status);
	Status = HeInitIoStatus(hDevice, &WriteIoStatus); // Initialise write IoStatus
	if(Status != HE_OK) error("Can't init Write I/O, exiting\n", Status);

	/* check that the selected slot has the right type of module in it */
	info[0]=HE_Get_ModuleTypes;
	Status = HeGetBoardInfo(devname,Board,info);
	if (Status!=HE_OK) error("error from get board info\n",Status);
		
	if (info[(5*(slotnumber-1))+1] ==0) 
	{
		sprintf(tmp,"error there is no module in slot %d\n",slotnumber);
		error(tmp,-1); /* no module*/
	}
	if (info[(5*(slotnumber-1))+2] ==1) 
	{
		sprintf(tmp,"error the module in slot %d is a processor module\n",slotnumber);
		error(tmp,-1); 
	}
	if (info[(5*(slotnumber-1))+3] ==0) 
	{
		sprintf(tmp,"error the module in slot %d does not have a serial bus\n",slotnumber);
		error(tmp,-1); 
	}

	/* if its an HEPC9 then set the HEART connections so that one host FIFO     */
	/* connects to each module slot. Then the host FIFO number determines which */
	/* module is being used */
	if(stricmp(devname,"hep9a")==0)
	{
		string=getapivar("HEAPI_DIR");
		/* check to see is a network file exists in this directory*/
		fptr=fopen("network", "r");	
		if (fptr!=NULL)
		{
			fclose(fptr);
#if _LINUX
			sprintf(command,"/usr/local/bin/heartconf -v network\n",string);
#else
			sprintf(command,"%s\\heartconf\\heartconf network\n",string);
#endif
		}
		else 
		{
			/* if not then look in the directory above in case we are running from a debig or release directory*/ 
#if _LINUX
			fptr=fopen("../network", "r");
#else
			fptr=fopen("..\\network", "r");
#endif
			if (fptr!=NULL)
			{
				fclose(fptr);
#if _LINUX
				sprintf(command,"/usr/local/bin/heartconf -v ../network\n",string);
#else
				sprintf(command,"%s\\heartconf\\heartconf ..\\network\n",string);
#endif
			}
			else
			{
				error("ERROR cannot locate the network file in this or the directory above\n",-1);
				getchar();
				exit(0);
			}
		}
		/* call heart conf to set the network up */		
		system(command);
	}

	Status = Initialise_Host_HSB(devname,Board);
	if(Status != HE_OK) error("Can't init HSB, exiting\n", Status);

	/* send messages over the HSB to set the registers in the FPGA that select the HERON FIFO */
	/* that the FPGA will use to read and write */
	fifo_in=1<<fifo;
	SendUserData(devname, Board, slotnumber, 0, fifo_in);

	fifo_out= 1<<fifo;
	SendUserData(devname, Board, slotnumber, 1, fifo_out);


#if FFT
	{
		float del_theta;
		int int_del_theta;

		del_theta = ( (ddsfreq/SAMPLE_FREQ)*pow(2,LUT_ADDR_BITS)) * pow(2,(TOT_ACC_BITS-LUT_ADDR_BITS));
		int_del_theta = (int)del_theta;
     
		SendUserData(devname, Board, slotnumber, 5 ,( int_del_theta&0xff) ); 
		SendUserData(devname, Board, slotnumber, 6 ,((int_del_theta>>8)&0xff)); 	
		SendUserData(devname, Board, slotnumber, 7 ,((int_del_theta>>16)&0xff)); 
		SendUserData(devname, Board, slotnumber, 8 ,((int_del_theta>>24)&0x03));
	}

	/* initialise signal source*/
	SendUserData(devname, Board, slotnumber, 4,fft_reg);
#else
#if DDC
	/* initialise signal source*/
	function4(hWnd);
	SendUserDatanr(devname, Board, slotnumber, 4,ddc_reg);

	/* write a AM and FM signals to the DAC rams in the FPGA*/
	for (i=0;i<512;i++)
	{
		phi_c_fm = ((float)((i*11) % 512) / 512) *(2*PI);
		phi_c_am = ((float)((i*20) % 512) / 512) *(2*PI);
		phi_m_fm    = ((float)((i) % 512) / 512) *(2*PI);
		phi_m_am    = ((float)((i*2 ) % 512) / 512) *(2*PI);
		/* AM in channel A data */
		chadata[i]=0x10000 |(0x3fff & (int)(0xfff*(0.5* (1+AM_INDEX*sin(phi_m_am))*sin(phi_c_am) ))); 
		/* FM in channel B data */ 
		chbdata[i]=0x40000 |(0x3fff & (int)(0xfff*( cos( phi_c_fm + FM_INDEX*sin(phi_m_fm)) )));
	} 

	// Start write to HERON module
	// Check if write started OK; if not, error
	Status = HeWrite(hDevice, chadata, (DAC_BLOCK)*sizeof(HE_DWORD), WriteIoStatus);
	if(Status == HE_IoInProgress) 	HeWaitForIo(hDevice, WriteIoStatus); 

	// Start write to HERON module
	// Check if write started OK; if not, error
	Status = HeWrite(hDevice, chbdata, (DAC_BLOCK)*sizeof(HE_DWORD), WriteIoStatus);
	if(Status == HE_IoInProgress) 	HeWaitForIo(hDevice, WriteIoStatus); 
#else
	/* initialise DAC arrays*/
	function1(hWnd);
#endif
#endif

	while (stopthread  == 0)
	{

#if DDC
		// send new value to the ddc mux
		if (controlsem==1)
		{
			SendUserDatanr(devname, Board, slotnumber, 4,ddc_reg);
			controlsem=0;
		}
#else
#if FFT
		// send new value to the ddc mux
		if (controlsem==1)
		{
			SendUserData(devname, Board, slotnumber, 4, fft_reg);
			controlsem=0;
		}
		if (controlsem==2)
		{
			float del_theta;
			int int_del_theta;

			del_theta = ( (ddsfreq/SAMPLE_FREQ)*pow(2,LUT_ADDR_BITS)) * pow(2,(TOT_ACC_BITS-LUT_ADDR_BITS));
			int_del_theta = (int)del_theta;
     
			SendUserData(devname, Board, slotnumber, 5 ,( int_del_theta&0xff) ); 
			SendUserData(devname, Board, slotnumber, 6 ,((int_del_theta>>8)&0xff)); 	
			SendUserData(devname, Board, slotnumber, 7 ,((int_del_theta>>16)&0xff)); 
			SendUserData(devname, Board, slotnumber, 8 ,((int_del_theta>>24)&0x03));
			controlsem=0;
		}
		if (controlsem==3)
		{
			fft_reg = fft_reg & (~4);
			fft_reg = fft_reg + (fftwindow<<2);
			SendUserData(devname, Board, slotnumber, 4, fft_reg);
			controlsem=0;
		}
#else
		/* if semaphore is set update DAC RAMs */
		if (controlsem==1)
		{
			// Start write to HERON module
			// Check if write started OK; if not, error
			Status = HeWrite(hDevice, chadata, (DAC_BLOCK)*sizeof(HE_DWORD), WriteIoStatus);
			if(Status == HE_IoInProgress) 	HeWaitForIo(hDevice, WriteIoStatus); 

			// Start write to HERON module
			// Check if write started OK; if not, error
			Status = HeWrite(hDevice, chbdata, (DAC_BLOCK)*sizeof(HE_DWORD), WriteIoStatus);
			if(Status == HE_IoInProgress) 	HeWaitForIo(hDevice, WriteIoStatus); 
			controlsem=0;
		}	
#endif
#endif
		// Start read from HERON module
		Status = HeRead(hDevice, results, (ADC_BLOCK)*sizeof(HE_DWORD), ReadIoStatus);

		SendUserDatanr(devname, Board, slotnumber, 2 ,((ADC_BLOCK)&0xff)); /* bottom part of count */
		SendUserDatanr(devname, Board, slotnumber, 3 ,(((ADC_BLOCK)>>8)&0xff)); /* top part of count and request to start*/	

		// Wait till Read completes
		HeWaitForIo(hDevice, ReadIoStatus);

#if _LINUX
		if (data_free)
		{
		   data_free = 0;
#endif
#if FFT
		   /* split the two channels*/
		   for (i=0;i<ADC_BLOCK;i++)
		   {
		   	data1[i]=(short)(results[i]&0xffff);
			data2[i]=(short)(results[i]>>16&0xffff);
		   }

		   for (i=0;i<ADC_BLOCK;i++)
		   {
			/* Calculate the magnitude of the FFT output */
			treal   = (double)data1[i];
			timag   = (double)data2[i];
			trealsq = treal*treal;
			timagsq = timag*timag;     
			sum_sqs = (trealsq + timagsq);
			if(sum_sqs <= 0.0) sum_sqs = 1.0;
			 /* the max magnitude will be 1.4 times max data[i] */
			/* but doubled the range because always +ve. (2 * 1/1.4) = 1.4 */
			data1[i] = (short)(1.4 * sqrt(sum_sqs) ) - 8191;
		   }

		   params.buf1     = data1;
		   params.bufsize1 = ADC_BLOCK;
		   params.buf2     = data1;
		   params.bufsize2 = ADC_BLOCK;
#else
		   /* split the two channels*/
		   for (i=0;i<ADC_BLOCK;i++)
		   { 
#if DDC
#if IO5
			data1[i]=((short)(results[i]&0xffff)/4);
		/*	data1[i]=data1[i]<<2;                   */
		/*	data1[i]=data1[i]>>2;                   */
			data2[i]=((short)(results[i]>>16&0xffff)/4);
		/*	data2[i]=data2[i]<<2;                   */
		/*	data2[i]=data2[i]>>2;                   */
#else
			// IO2
			data1[i]=(short)(results[i]&0x3fff);
			data1[i]=data1[i]<<2;
			data1[i]=data1[i]>>2;
			data2[i]=(short)(results[i]>>16&0x3fff);
			data2[i]=data2[i]<<2;
			data2[i]=data2[i]>>2;
#endif
#else
#if IO5
			data1[i]=(short)(results[i]&0xfff);
			data1[i]=data1[i]<<4;
			data1[i]=data1[i]>>4;
			data2[i]=(short)(results[i]>>16&0xfff);
			data2[i]=data2[i]<<4;
			data2[i]=data2[i]>>4;
#else
			data1[i]=(short)(results[i]&0xfff);
			data1[i]=data1[i]<<4;
			data1[i]=data1[i]>>4;
			data2[i]=(short)(results[i]>>16&0xfff);
			data2[i]=data2[i]<<4;
			data2[i]=data2[i]>>4;
#endif
#endif
		   }

		   params.buf1     = data1;
		   params.bufsize1 = ADC_BLOCK;
		   params.buf2     = data2;
		   params.bufsize2 = ADC_BLOCK;
#endif

#if _LINUX
                   //DrawWindow(ADC_BLOCK, data1);
		   data_display = 1;
		}
#else
		SendMessage(hWnd, HE_UPDATEGRAPH, (unsigned int) &params, 0);
#endif
		
	} /* while */

	/* Close down API and return */
	Status = HeClose(&hDevice);
	if(Status != HE_OK) error("Failed close..., exiting\n", Status);
	// Close down HSB device
	Status = HeClose(&hsbDevice);
	if(Status != HE_OK) error("Failed close..., exiting\n", Status);
}

void function1(HWND hWnd)
{
#if FFT
	/* set to ADC  output */
	fft_reg = 8 + ((fftwindow&1)<<2);
	controlsem=1;
#else
#if DDC
#if IO5
	/* set to fir filt2 output */
	ddc_reg = 4;
	controlsem=1;
#else
	/* set to ADC and DDS output */
	ddc_reg = 0;
	controlsem=1;
#endif
#else
	int i;
#if IO5
	for (i=0;i<512;i++)
	{
		chadata[i]=0x10000 | (0xffff & (int)(0x7fff*( sin(i*(PI/0x10))))); 
		chbdata[i]=0x40000 | (0xffff & (int)(0x7fff*( cos(i*(PI/0x10)))));          
	}
#else
	for (i=0;i<512;i++)
	{
		chadata[i]=0x10000 |(int)(0x1fff+ 0x1fff*( sin(i*(PI/0x10)))); 
		chbdata[i]=0x40000 |(int)(0x1fff+ 0x1fff*( cos(i*(PI/0x10))));          
	}
#endif
	/* set semaphore to schedule DAC update*/
	controlsem=1;
#endif
#endif
}

void function2(HWND hWnd)
{
#if FFT
	/* set to DDS output */
	fft_reg = 9 + ((fftwindow&1)<<2);
	controlsem=1;
#else
#if DDC
#if IO5
	/* set to fir filt2 output */
	ddc_reg = 4;
	controlsem=1;
#else
	/* set to multiplier output */
	ddc_reg = 1;
	controlsem=1;
#endif
#else
	int i;
#if IO5
for (i=0;i<512;i++)
{
	if ((i&0x10)==0)
	{
		chadata[i]=0x10000 |(int)(0x8000); 
		chbdata[i]=0x40000 |(int)(0x8000);          
	}
	else
	{
		chadata[i]=0x10000 |(int)(0x7fff); 
		chbdata[i]=0x40000 |(int)(0x7fff);
	}
}
#else
	for (i=0;i<512;i++)
	{
		if ((i&0x10)==0)
		{
			chadata[i]=0x10000 |(int)(0); 
			chbdata[i]=0x40000 |(int)(0);          
		}
		else
		{
			chadata[i]=0x10000 |(int)(0x3fff); 
			chbdata[i]=0x40000 |(int)(0x3fff);
		}
	}
#endif
	/* set semaphore to schedule DAC update*/
	controlsem=1;
#endif
#endif
}

void function3(HWND hWnd)
{
#if FFT
	/* set to ADC  output */
	fft_reg = 8 + ((fftwindow&1)<<2) ;
	controlsem=1;
#else
#if DDC
#if IO5
	/* set to fir filt2 output */
	ddc_reg = 4;
	controlsem=1;
#else
	/* set to cic output */
	ddc_reg = 2;
	controlsem=1;
#endif
#else
	/* we must provide a blank function at least */
#endif
#endif
}

void function4(HWND hWnd)
{
#if FFT
	/* set to ADC  output */
	fft_reg = 8 + ((fftwindow&1)<<2) ;
	controlsem=1;
#else
#if DDC
#if IO5
	/* set to fir filt2 output */
	ddc_reg = 4;
	controlsem=1;
#else
	/* set to fir filt2 output */
	ddc_reg = 4;
	controlsem=1;
#endif
#else
	/* we must provide a blank function at least */
#endif
#endif
}

#if _LINUX
void error(char *string, int Status)
{
   if (Status >= 0)
   {
      printf("Status=%08lx. %s", Status, string);
   }
   else
   {
      printf("%s", string);
   }
   Status = HeClose(&hDevice);
   exit(0);
}
#else
void error(char *string, int Status)
{
   MessageBox(NULL, string, "Fatal Error", MB_OK);
   Status = HeClose(&hDevice);
   exit(0);
}
#endif

void SendUserData(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data)
{
	HE_BYTE user[2];
	HE_BYTE rec_data[3];
	int reply;
	char string[255];

	// Send user data
	user[0]=(HE_BYTE) Address;
	user[1]=(HE_BYTE) Data;

	Status = HeHSBSendMessage(hsbDevice, USER_WRITE, slot, &user[0], 2, 1000);
	if (Status != HE_OK)
	{
		sprintf(string,"Write module failed  %x :board-%x: slot-%x, \n", Status,Board,slot);
		if (Status != HE_OK) error(string, Status);
	 }
	// read it back to check
	reply = readback_user(slot,rec_data,3,Address);
	if (reply!=Data)
	{
		sprintf(string,"error hsb readback failed %x: board-%x: slot-%x (read %x, expected %x)\n", reply,Board,slot,reply,Data);
		error(string,-1); 
	}
}

void SendUserDatanr(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data)
{
	HE_BYTE user[2];
	char string[255];

	// Send user data
	user[0]=(HE_BYTE) Address;
	user[1]=(HE_BYTE) Data;

	Status = HeHSBSendMessage(hsbDevice, USER_WRITE, slot, &user[0], 2, 1000);
	if (Status != HE_OK)
	{
		sprintf(string,"Write module failed %x: board-%x: slot-%x\n", Status,Board,slot);
		if (Status != HE_OK) error(string, Status);	
	}
}


/********************************************************************************/
/*                                                                              */
/* Initialise_Host_HSB                                                          */
/*                                                                              */
/* This function sets up the host HSB interface of each board in the system.    */
/*                                                                              */
/********************************************************************************/
int Initialise_Host_HSB(char *dev_name, HE_WORD board)
{
	HE_DWORD switches;
	
	hsbDevice=NULL;


	/* Check parameters */
	if ((board > 15)) {
		error("Error: board ID is out of range (0 to 15)\n",-1);
		return(-1);
	}

	switches = HE_Switch_Last;
	Status = HeOpenS(dev_name, board, "hsb\0", &hsbDevice, &switches);
	if (Status != HE_OK) return(1);


	return(0);
}



/********************************************************************************/
/*                                                                              */
/* readback_user                                                                */
/*                                                                              */
/* This function is used to read back from the FPGA slot                        */
/*                                                                              */
/********************************************************************************/
int readback_user(HE_BYTE slot, HE_BYTE *reply, int buffer_size, HE_BYTE user_addr)
{
	HE_BYTE  message_type;
	HE_DWORD bytes_read;
	HE_BYTE  send_data[5];

	send_data[0]=user_addr;
	send_data[1]=1;



	Status = HeHSBSendMessage(hsbDevice, USER_READ, slot, send_data, 2, 1000);
	Status = HeHSBReceiveMessage(hsbDevice, &message_type, slot, reply, buffer_size, &bytes_read, 1000);
	if (Status == HE_OK) 
	{
		if (message_type != USER_READ_REPLY) 
		{
			error("Error: expected user read reply\n",-1);
			return(-1);
		}
		else 
		{

			return(reply[0]);
		}
	}

	return(Status);
}



